#!/usr/bin/env python
from optparse import OptionParser
import template
import string
import logging
import handlers
from expparser import ExpParser
from safe.helpers import TupleIndexer
from safe.dpgenerators import *

class Templator(object):

    def __init__(self, template_string, ned_string):
        self.template = template.CreateFromDocument(template_string)
        p = ExpParser(ned_string)
        p.parse()
        self.design_points = LinearDesignPointGenerator(p.get_factors_levels(),
        p.get_restriction_expression())

        divisor = 1
        self.indexers = {}
        self.num_block_combos = 1
        for group in self.template.group:
            self.num_block_combos = self.num_block_combos * len(group.block)
            self.indexers[group] = TupleIndexer(divisor, len(group.block))

    def apply_templates(self, block_set, dp):
        script_set = []
        for block in block_set:
            for factor in dp.keys():
                placeholder = '$%s$' % factor
                block = string.replace(block, placeholder, str(dp[factor]))
            script_set.append(block)
        return script_set

    def get_scripts(self):
        for dp in self.design_points:
            for master_index in range(0, self.num_block_combos):
                block_set = []
                for group in self.template.group:
                    index = self.indexers[group].index(master_index)
                    b = group.block[index]
                    block_set.append(b)
                script_set = self.apply_templates(block_set, dp)
                # s = self.apply_template(block_set, dp)
                # yield {'block_set' : block_set, 'dp' : dp}
                yield script_set
    
    def get_all_scripts(self):
        
        for dp in self.design_points:
            for master_index in range(0, self.num_block_combos):
                block_set = []
                for group in self.template.group:
                    index = self.indexers[group].index(master_index)
                    b = group.block[index]
                    block_set.append(b)

if __name__ == '__main__':
    usage = './%prog EXPERIMENT_DESCRIPTION.xml EXPERIMENT_TEMPLATE.xml [options]'
    parser = OptionParser(usage = usage)
    parser.add_option('-v', '--verbose', dest='verbose',
                        action='store_true', default=False,
                        help='print verbose output')
    (options, args) = parser.parse_args()


    logger = logging.getLogger('templator_logger')
    logger.setLevel(logging.DEBUG)

    if (options.verbose):
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        logger.addHandler(ch)
    else:
        nh = handlers.NullHandler()
        nh.setLevel(logging.DEBUG)
        logger.addHandler(nh)

    ned_string = file(args[0]).read()
    template_string = file(args[1]).read()
    t = Templator(template_string, ned_string)
    g = t.get_scripts()

    script_set = g.next()
    for s in script_set:
        print(s)

    script_set = g.next()
    for s in script_set:
        print(s)

    script_set = g.next()
    for s in script_set:
        print(s)

    script_set = g.next()
    for s in script_set:
        print(s)
