#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: William Stratton (wss012@bucknell.edu)
#

"""
This command creates a globally visible "installation" in the SAFE server. An installation
contains the source code of a full ns-3 distribution, which needs to be configured and
compiled before it is used. The command is available exclusively through the command-line
interface and should not be used by novice users. It will be useful for one who
wants to set up an experiment that a group of novice users can use. For instance,
an instructor who wants his class to experiment with one same simulation scenario
would use this.
"""

import argparse
import requests
import common
import os
import json
import sys
#import getpass

def get_owner_id(owner_name, config):
    """Get the list of current users from the database, and check them against the owner argument """
    
    getheaders = {}
    getheaders['Authorization'] = 'Token ' + config['token']
    #Use JSON to more easily parse the results                                                                                                                                                                     
    getheaders['accept'] = 'application/json'

    r = requests.get(config['base-url'] + "users/", headers=getheaders)

    if r.status_code != 200 :
        print "Error getting list of users"
        return None

    for i,x in enumerate(r.json()['results']):
        if x['username'] == owner_name:
            return config['base-url'] + 'users/'+str(i+1)+'/'
        
    print "User '" + owner_name + "' not found, please specify a valid owner"
    return None


def main():
    version = '0.1'
    copyright = '(c) Bucknell University 2013'
    epilog_str = copyright + ', version ' + version 

    parser = argparse.ArgumentParser(
        add_help=True,
        description='Creates an experiment installation for this user on the SAFE server.')
    parser.add_argument( '-f', '--file',
        required=True, 
        help='path of tarball with installation')
    parser.add_argument('-p', '--public',
        action='store_true',
        help = 'is the installation public?')
    parser.add_argument( '-n', '--name',
        required=True,
        help='name of the installation')
    parser.add_argument( '-d', '--description',
        required=True,
        help='description of the installation')
    parser.add_argument( '-o', '--owner',
        default = None,
        help='username of installation owner')

    args = parser.parse_args()

    
    print ">>> creating new experiment installation on SAFE"

    config = common.get_config()

    if args.owner is None:
        args.owner = config['username']

    data = {}

    data['owner'] = get_owner_id(owner_name, config)
    if data['owner'] == None:
        return 1

    
    data['public'] = args.public
    data['name'] = args.name
    data['description'] = args.description
    
    files = {'tarball_path': open(os.path.abspath(args.file), 'rb')}
   
    header = {}
    header['Authorization'] = 'Token ' + config['token']

    r = requests.post(config['base-url'] + "installations/",  headers=header,  data = data, files = files)

    files['tarball_path'].close()

    if r.status_code == 201:
      print "Success, id: "+str(json.loads(r.text)['id'])
      return 0

    print "Error"
    return 1

if __name__ == "__main__":
    if main() != 0:
        sys.exit(1)
