#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2014 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: Nicholas Marshall (nam021@bucknell.edu) based off previous work by William Stratton (wss012@bucknell.edu)
#

""" This command Generates an experiment config file skeleton for a user to add and define levels and names for. Filetype is JSON. """

import common
import requests

def main():
    import argparse
    version = '0.1'
    copyright = '(c) Bucknell University 2013'
    epilog_str = copyright + ', version ' + version 

    parser = argparse.ArgumentParser(
            add_help=True,
            description='Generate Config File Skeleton',
            epilog=epilog_str)
       
    parser.add_argument('--desc_id','-e',
            required=True,
            help='Experiment Description id')
    
    parser.add_argument('--replications','-r',
            required=False,
            default=1,
            help='Number of replications')
    
    parser.add_argument('--description','-d',
            required=False,
            default="",
            help='Description of experiment')
    
    parser.add_argument('--name','-n',
            required=True,
            help='Name of experiment')
    
    parser.add_argument('--public','-p',
            required=False,
            default="true",
            help='Whether or Not experiment should be public')
    
    parser.add_argument('filename',
            help='name of config file')
                        
    args = parser.parse_args()

    config = common.get_config()
    
    header = {}
    header['Authorization'] = 'Token ' + config['token']        
    header['content-type'] = 'application/json'
    
    r = requests.get(config['base-url'] + 'exp-desc/'+str(args.desc_id)+'/factors/', headers = header)
    if r.status_code >= 400:
        print("Error retrieving factors")
        return 1
    
    factors = r.json()

    json_data = {
        'replications': args.replications,
        'description': args.description,
        'exp_description_id': args.desc_id,
        'name': args.name,
        'public': args.public,
        'exp_description_id': args.desc_id 
    }

    for factor in factors:
        factor.pop('id', None)
        factor['levels'] = []
        
    json_data['factors'] = factors
    
    new_filename = args.filename
    if new_filename[-5:] != '.json':
        new_filename = new_filename + '.json'
        
    with open(new_filename, 'w') as outfile:
        import json    
        json.dump(json_data, outfile, indent=2)
    print("Success")
    return 0

if __name__ == "__main__":
    if main() != 0:
        sys.exit(1)
