#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: William S. Stratton (willstrat@gmail.com)
#

""" This command queries the SAFE server for the factors for an Experiment or Experiment Description. The user must specify the id of the Experiment or Experiment Description that they want the factors from. The user may also specify the format of the data (default json)
"""

import common

def main():
    import argparse
    version = '0.1'
    copyright = '(c) Bucknell University 2013'
    epilog_str = copyright + ', version ' + version 

    parser = argparse.ArgumentParser(
            add_help=True,
            description='Dump data to a file.',
            epilog=epilog_str)
    
    parser.add_argument('--exp_id','-e',
            required=False,
            help='Experiment id')
    
    parser.add_argument('--desc_id','-d',
            required=False,
            help='Experiment Description id')

    parser.add_argument('--filetype','-f',
            required=False,
            default='json',
            help='file type')

    parser.add_argument('output_file',
                        nargs="?",
                        help='name of dump file')                        

    args = parser.parse_args()
    if args.exp_id is None  and args.desc_id is None:
        parser.error("One of experiment id (-e) or description id (-d) required")
    if args.exp_id is not None  and args.desc_id is not  None:
        parser.error("Only one of of experiment id (-e) or description id (-d) should be supplied at a time")
    e_id = args.exp_id if args.desc_id is None else args.desc_id
    
    print ">>> dumping factors from "+("experiment " if args.exp_id is not None else "experiment description ") + e_id + " to "+ ("stdout" if args.output_file is None else "a " + args.filetype + " file named " + args.output_file )

    url_start = 'exp' if args.desc_id is None else 'exp-desc'
    json = fetch_data(url_start, e_id)

    dumpdata(json, args.filetype, args.output_file)

def fetch_data(url_start, e_id):
    import requests
    config = common.get_config()
    header = {}
    header['Authorization'] = 'Token ' + config['token']        
    header['content-type'] = 'application/json'
    r = requests.get(config['base-url'] + url_start+'/'+str(e_id)+'/factors/', headers = header)
    return r.json()

def dumpdata(json, filetype, filename):
    if filetype == 'csv':
        dump_csv(json, filename)
    else:
        dump_json(json, filename)

def dump_json(json_data, filename):
    import json
    if filename is None:
        print json.dumps(json_data, indent=4)
    else:
        new_filename = filename
        if new_filename[-5:] != '.json':
            new_filename = new_filename + '.json'
        outfile = open(new_filename, 'w')
        json.dump(json_data, outfile)

def dump_csv(json, filename):
    if filename is None:
        import json
        print "Error: output_file not specified, printing to stdout as json"
        print json.dumps(json_data, indent=4)
    else:
        import csv
        new_filename = filename
        if new_filename[-4:] != '.csv':
            new_filename = new_filename + '.csv'
        outfile = open(new_filename+'.csv', 'w')
        header = json[0].keys()
        writer = csv.DictWriter(outfile, header)
        writer.writeheader()
        map(writer.writerow, json)
        outfile.close()
 
if __name__ == "__main__":
    main()
