#!/usr/bin/env python
## -*- Mode: python; py-indent-offset: 4; indent-tabs-mode: nil; coding: utf-8; -*-
#
# Copyright (c) 2013 Bucknell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation;
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Author: L. Felipe Perrone (perrone@bucknell.edu)
#
import requests
import argparse
import os
import sys
import json
import common
""" This command startes the execution of a previously configured experiment within an installation on SAFE.
"""

def main():
    version = '0.1'
    copyright = '(c) Bucknell University 2013'
    epilog_str = copyright + ', version ' + version 

    parser = argparse.ArgumentParser(
            add_help=True,
            description='Start the execution of a configured experiment.',
            epilog=epilog_str)
    parser.add_argument('--expid','-e',
            required=False,
            default=None,
            help='experiment id')
    parser.add_argument('--expname','-n',
            required=False,
            default=None,
            help='experiment name')
    parser.add_argument('--min_results','-m',
            required=False,
            default=None,
            help='minimim number of results before checking precision')
    parser.add_argument('--conf_level','-c',
            required=False,
            default=None,
            help='confidence level to use when calculating precision')
    parser.add_argument('--server_port','-p',
            required=False,
            default=None,
            help='port that server process should run on')
    parser.add_argument('--analyzer_port','-a',
            required=False,
            default=None,
            help='port that global analyzer process should run on')
    args = parser.parse_args()

    if args.expname is None and args.expid is None:
        print "One of expid or expname is required"
        return 1
    opts = dict(vars(args))
    opts.pop('expid', None)
    opts.pop('expname', None)
    options = { k : opts[k] for k in opts if opts[k] != None }
    config = common.get_config()
    header = {}
    header['Authorization'] = 'Token ' + config['token']

    if args.expid is None:
        r = requests.get(config['base-url'] +"view_experiments/", data = {}, headers=header)
        if int(r.status_code)  != 200:
            print "Not authenticated, please log in before starting an experiment"
            return 1
        for e in r.json():
            if e['exp_name'] == args.expname:
                args.expid = str(e['exp_id'])
                break
        else:
            print "Experiment "+str(args.expname)+" Not found, please specify a valid experiment owned by the current user"
            sys.exit(1)
    arg_string = "?"
    for key, value in options.iteritems():
        amp = "&" if arg_string[-1] != "?" else ""
        arg_string += amp + key + "=" + str(value)
    r = requests.get(config['base-url'] +"exp/"+args.expid+"/run/", data = {}, headers=header)
    
    print r.text
    return 0


if __name__ == "__main__":
    exit_code = main()
    if exit_code != 0:
        sys.exit(exit_code)


