// Layout components
var jMarginFocus, jMarginContext, sFocusWidth, sContextWidth, sFocusHeight, sContextHeight;
// Scaling 
var dXScaleFocus, dYScaleFocus, dXScaleContext, dYScaleContext;
// Axes to be used for the graph
var xAxisFocus, yAxisFocus, xAxisContext, yAxisContext;
// Areas where things will be graphed
var svg, context, focus, brush;
// Our data, this will need to change
var designPointList, factorList,  runList, metricList, metricData, metricStatistics, aDisplayList, levelList, parameterList;
// Inportant details about the data
var xValMin, xValMax, yValMin, yValMax;
// The name of the independent variable / dependent variables (Array)
var aDisplayList, aDisplayColor, aColors;
// Drawing details
var contextDataPointRadius, focusDataPointRadius;
// Used to display data information
var infoMessage, numberFormat;
var exp_id;

// Initializes graph componenets and variables
function initVisualComponents(){
	// Configure margins
    jMarginFocus = {top: 40, right: 40, bottom: 20, left: 40};
	jMarginContext = { top: 20, right: jMarginFocus.right, bottom: 20, left: 40 };
	sFocusWidth = 960 - jMarginFocus.left - jMarginFocus.right;
	sFocusHeight = 500 - jMarginFocus.top - jMarginFocus.bottom;
	sContextWidth = sFocusWidth;
	sContextHeight = 150 - jMarginContext.top - jMarginContext.bottom;

    focus = new LineMagnitude(jMarginFocus, 960, 500, "chart1");
    context = new LineMagnitude(jMarginContext, 960, 150, "chart2", true);

    brush = d3.svg.brush() 
        .x(context.dXScale)
        .on("brush", brushed);

    context.svg.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + jMarginContext.left + "," + jMarginContext.top + ")")
        .call(brush)
        .selectAll("rect")
            .attr("y", -6)
            .attr("height", sContextHeight + 7);

    // Add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

	// Miscilanious variables
	aDisplayColor = {};
    aDisplayList = {};
	metricData = {};
	metricStatistics = {};
    aColors = ["blue", "green", "orange", "purple", "black", "red"];
	contextDataPointRadius = 1.5;
	focusDataPointRadius = 3;
	parameterList = {};
    numberFormat = d3.format(".2f");
}

// Initializes the selection for the graph
function initGraphSelect(){
    // Initialize the factor list
    d3.selectAll("#select")
        .append("div")
        .append("select")
        .attr("class", "key_line")
        .attr("id", "selectFactor")
        .on("change", factorSelectChange);

    d3.select("#selectFactor")
        .selectAll("option")
        .data(Object.keys(factorList))
        .enter()
        .append("option")
        .attr("factor", function(d) { return d } )
        .text(function(d) { return factorList[d].name });

    // Initialize metric selects
    var select_items = d3.select("#select").selectAll(".select_line").data(Object.keys(metricList)).enter()
        .append("div")
        .attr("class", "select_line")
        .attr("metric_id", function(d) { return metricList[d].id })
        .attr("id", function(d) { return "select_" + metricList[d].id });

    select_items.append("div")
        .attr("class", "select_square")
        .attr("id", function(d) { return "select_square_" + metricList[d].id });

    select_items.append("div")
        .attr("class", "select_label")
        .attr("id", function(d) { return "select_label_" + metricList[d].id })
        .text(function(d) { return metricList[d].name});

    d3.selectAll(".select_line")
        .on("click", selectElementChange);

    // Initialize the cross hairs option
    var crossHairEnable = d3.selectAll("#options")
        .append("div")
        .attr("id", "crossHairEnable");

	crossHairEnable.append("input")
        .attr("type", "checkbox")
        .style("float", "left")
        .on("change", enableCrossHairs);

	crossHairEnable.append("p")
        .text("Cross Hairs");
}

// Reads json data from the given url
// When it's loaded the given callBack function
// is called with the resulting data
function readData(url, callBack){
    d3.json(url, function(error, json){
        if (error) return console.warn(error);

        callBack(json);
    });
}

// Reads metric information
function readInitialData(){
    designPointList = [];
    runList = [];
    factorList = {};
    levelList = {};
    metricList = {};
 
    getDesignPoints = function(data){
        d3.json("/safe/exp/" + exp_id + "/designpoints/", function(error, designPoints){
            if(error){ console.warn(error) }
            designPoints.forEach(function(designPoint){ designPointList[designPoint.id] = designPoint });
            designPoints.forEach(function(designPoint){ getSimrunList(designPoint) });
            getFactors();
        });
    }
  
    getSimrunList = function(designPoint){
        d3.json("/safe/exp/" + exp_id + "/designpoints/", function(error, simruns){
            if(error){ console.warn(error) }
            runList.concat(simruns);
            designPoint["runs"] = simruns;
        });
    }
  
    getFactors = function(){
        d3.json("/safe/exp/" + exp_id + "/factors/", function(error, factors){
            factors.forEach(function(factor){ factorList[factor.id] = factor });
            getLevels();
        });
    }
  
    getLevels = function(){
        d3.json("/safe/exp/" + exp_id + "/levels/", function(error, levels){
            Object.keys(factorList).forEach(function(factor_id){
                levelList[factor_id] = levels[factor_id].levels.sort();
            });
            getMetrics();
        });
    }
 
    getMetrics = function(){
        d3.json("/safe/exp/" + exp_id + "/metrics/", function(error, metrics){
            metrics.forEach(function(metric){
                metricList[metric.id] = metric;
            });
            initGraphSelect();
            displayData();
        });
    }

    getDesignPoints();
}


// displays the currently held data to the graph
function displayData(){
	context.displayAll();
}
// updates the positions of the center points described by the data groupings
function updateContextDataPoints(factor_id, metric_id){
    
}

function updateFocusDataPoints(factor_id, metric_id){
    focus.displayAll();
}

function addDisplayMetric(factor_id, metric_id){
    var eFactorSelect = document.getElementById("selectFactor")
    var sSelectedFactor = eFactorSelect.options[eFactorSelect.selectedIndex].getAttribute("factor");
    if(factor_id == sSelectedFactor){ d3.selectAll("#select_square_" + metric_id).style("background", aDisplayColor[factor_id][metric_id]); }

    aDisplayList[factor_id][metric_id] = true;
        
    if(metricStatistics[factor_id] == undefined){ metricStatistics[factor_id] = {} }
    if(metricStatistics[factor_id][metric_id] == undefined){
        var simrunList = [];
        
        var getFactorDesignPoints = function(level){
            d3.json("/safe/dp-query/?factor=" + factorList[factor_id].name + "&exp_id=" + exp_id + "&value=" + level, function(error, designPoints){
                if(error){ console.warn(error) }
                designPoints.forEach( function(designPoint, designPointIndex){
                    getDesignPointSimruns(designPoint, level, designPointIndex, designPoints.length - 1);
                });
            });
        }

        var getDesignPointSimruns = function(designPoint, level, currentIndex, maxIndex){
            d3.json("/safe/designpoint/" + designPoint.designpoint_id + "/simruns/", function(runs) {
                simrunList.push.apply(simrunList, runs);
                if(currentIndex >= maxIndex)
                    getMetricResults(level);
            });
        }

        var getMetricResults = function(level){
            d3.json("/safe/metric/" + metric_id + "/results/").post(JSON.stringify(simrunList), function(error, simruns){
                if(error) { console.warn(error) }
                
                calculateStatistics(factor_id, metric_id, level, simruns);
                if(Object.keys(metricStatistics[factor_id][metric_id]).length >= levelList[factor_id].length)
                    evaluateMetric();
            });
        }

        var evaluateMetric = function(){
            data = [];
            levelList[factor_id].forEach(function(level){
                data.push(metricStatistics[factor_id][metric_id][level]);
            });
            pointAttributes = { "stroke" : aDisplayColor[factor_id][metric_id],
                                "r" : contextDataPointRadius,
                                "stroke-width" : 1.5,
                                "fill" : "white",
                                "fill_opacity" : 0 }

            pointMouseEvents ={ "mouseover" : mouseOverDataPoint,
                                "mouseout" : mouseOutDataPoint }

            context.addDataSet(factor_id + "_" + metric_id, data, pointAttributes);

            pointAttributes["r"] = focusDataPointRadius;
            pointAttributes["info"] = function(d, e, f) { return numberFormat(currentData[f]["xValue"]) + "<br/>" + numberFormat(currentData[f][d]) } ;
            focus.addDataSet(factor_id + "_" + metric_id, data, pointAttributes, pointMouseEvents);
            
            displayData();
            brushed();
            updateKeyItems();
        }

        levelList[factor_id].forEach(function(level) { getFactorDesignPoints(level) });
    }else{
        focus.setVisible(factor_id + "_" + metric_id);
        context.setVisible(factor_id + "_" + metric_id);
        displayData();
		updateKeyItems();
    }
}

// removes the specified key from the list of dependent variables (no longer displayed)
function removeDisplayMetric(factor_id, metric_id){
    var eFactorSelect = document.getElementById("selectFactor")
    var sSelectedFactor = eFactorSelect.options[eFactorSelect.selectedIndex].getAttribute("factor");

    if(!focus.hasData(factor_id + "_" + metric_id) || !context.hasData(factor_id + "_" + metric_id)) { return }

    if(factor_id == sSelectedFactor){ d3.selectAll("#select_square_" + metric_id).style("background", "white"); }

    focus.setInvisible(factor_id + "_" + metric_id);
    context.setInvisible(factor_id + "_" + metric_id);
	
    delete aDisplayList[factor_id][metric_id];
    displayData();
	updateKeyItems();
}

// called when the mouse is positioned over a data element
function mouseOverDataPoint(){
	infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);
    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}

function brushed() {
    var extent = brush.extent();

    focus.updateDomains(brush.empty() ? context.dXScale.domain() : extent);

    Object.keys(aDisplayList).forEach(function(factor_id){
          Object.keys(aDisplayList[factor_id]).forEach(function(metric_id){
                updateFocusDataPoints(factor_id, metric_id);
        });
    });
}

function enableCrossHairs(){
    if(this.checked){
        focus.enableCrossHairs();
    }else{
        focus.disableCrossHairs();
    }
}

function factorSelectChange(){
    //console.log("Factor changed...");
	eFactorSelect = document.getElementById("selectFactor");
	sSelectedFactor = eFactorSelect.options[eFactorSelect.selectedIndex].getAttribute("factor");
	Object.keys(metricList).forEach(function(metric_id){
		d3.select("#select_square_" + metric_id).style("background", "white");
	});

	Object.keys(aDisplayList).forEach(function(factor_id){
		Object.keys(aDisplayList[factor_id]).forEach(function(metric_id){
			if(sSelectedFactor == factor_id){ 
				addDisplayMetric(factor_id, metric_id);
                aDisplayList[factor_id][metric_id] = true;
				d3.select("#select_square_" + metric_id).style("background", aDisplayColor[factor_id][metric_id]);
			}
			else{ 
                removeDisplayMetric(factor_id, metric_id);
                aDisplayList[factor_id][metric_id] = false;
                
            }
			});
		});
    focus.updateDataFeatures();
    context.updateDataFeatures();

    context.displayAll();
    brushed();
    displayData();
	updateKeyItems();
}

function selectElementChange(){
     //console.log("Selected Element: ", this.getAttribute("metric_id"));
    
     var eFactorSelect = document.getElementById("selectFactor");
	 var sSelectedFactor = eFactorSelect.options[eFactorSelect.selectedIndex].getAttribute("factor");
     var sSelectedMetric = this.getAttribute("metric_id");

     if(aDisplayColor[sSelectedFactor] == undefined){ aDisplayColor[sSelectedFactor] = {} }
     if(aDisplayList[sSelectedFactor] == undefined) { aDisplayList[sSelectedFactor] = {} }

     if(aDisplayColor[sSelectedFactor][sSelectedMetric] == undefined) { aDisplayColor[sSelectedFactor][sSelectedMetric] = aColors.shift() }

     if(aDisplayList[sSelectedFactor][sSelectedMetric] == undefined){
         addDisplayMetric(sSelectedFactor, sSelectedMetric);
     }else if(aDisplayList[sSelectedFactor][sSelectedMetric] == true){
         removeDisplayMetric(sSelectedFactor, sSelectedMetric);
     }else if(aDisplayList[sSelectedFactor][sSelectedMetric] == false){
         aDisplayList[sSelectedFactor][sSelectedMetric] = true;
     }
}

function displayRemove(){
    //console.log("Removing ", this, "...");
    removeDisplayMetric(this.getAttribute("factor_id"), this.getAttribute("metric_id"));
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
}

readParameters();
initVisualComponents();
readInitialData();
