/**
 * Functions for loading design points from the data base
 * in various ways
 */

safe.loader.designpoints = {};
safe.loader.designpoints.byFactorLevel = safe_loader_designpoints_byFactorLevel;
safe.loader.designpoints.byFactorRange = safe_loader_designpoints_byFactorRange;
safe.loader.designpoints.byExperiment = safe_loader_designpoints_byExperiment;

/**
 * Retrieves all design points which have the specified factor at the given level.
 * Data is of the form [{"designpoint_id":<designpoint_id>}, ...]
 *
 * @param experimentId The id of the experiment in which to search occur
 * @param factorName The name of the factor for which the level will be defined
 * @param factorLevel The level of the specified factor
 * @param callBack A function to be called after the data has been read.
 *          It is called in the form "callBack(data)".
 */
function safe_loader_designpoints_byFactorLevel(experimentId, factorName, factorLevel, callBack){
    d3.json("/safe/dp-query/?exp_id=" + experimentId + "&factor_name=" + factorName + "&value=" + factorLevel, function(error, data){
        if(error) { console.warn(error) }

        callBack(data);
    });
}

/**
 * Retrieves all design points with factor levels within the specified range
 * Data is of the form [{"designpoint_id":<designpoint_id>}, ...]
 *
 * @param experimentId The id of the experiment in which the search is to occur
 * @param factorList A list of factors and their ranges of the form [{name:<factor_name>, min_value:<min>, max_value:<max>}, ...]
 * @param callBack A function to be called after the data has been read.
 *          It is called in the form "callBack(data)"
 */
function safe_loader_designpoints_byFactorRange(experimentId, factorList, callBack){
    d3.json("/safe/exp/" + experimentId + "/dp-range-query/").post(JSON.stringify(factorList), function(error, data){
        if(error) { console.warn(error) }

        callBack(data);
    });
}

/**
 * Returns a list of all design points associated with the specified experiment
 * Data is of the form [{"id", <designpoint_id>, "factors":{<factor_name>:<factor_level>, ...} }, ... ]
 *
 * @param experimentId The experiment id for which to search
 * @param callBack A function to be called after the data has been read "callBack(data)"
 */
function safe_loader_designpoints_byExperiment(experimentId, callBack){
    d3.json("/safe/exp/" + experimentId + "/designpoints/", function(error, data){
        if(error) { console.warn(error) }

        callBack(data);
    });
}
