// The experiment currently being observed
var exp_id;
var colors = ["blue", "green", "orange", "purple", "black", "red"];
var focusDataPointRadius = 2;
var contextDataPointRadius = 1.5;
var numberFormat = d3.format(".2f");
var windowResizeFunctions = [];

var selectFactor;

// add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

//----------------------------------------------
// Apply Factor Selection Object
// ---------------------------------------------

selectionFactorApplyNumber = 0;
SelectionFactorApply = function(exp_id, applyCallBack, parentDomId){
	this.exp_id = exp_id;
	this.callBack = applyCallBack;
	this.dParent = d3.select("#" + parentDomId);
	this.eParent = document.getElementById(parentDomId);
	this.id = "selectionFactorApply_" + selectionFactorApplyNumber;
	selectionFactorApplyNumber += 1; 

	this.dSelf = this.dParent
        .append("div")
        .attr("id", this.id)
        .attr("class", "factor_select_header")
        
	//console.log(this.dSelf);
	this.selectFactor = new SelectionFactor(this.exp_id, undefined, this.id);
	this.selectFactor.readInitialData(this.initComponents.bind(this));
}

SelectionFactorApply.prototype.initComponents = function(){
    this.dSelf.append("div")
        .text("Select Graph to View")
        .attr("class", "plot_area_label");
	this.selectFactor.initComponents({"class":"half_line"});
	this.dSelf
		.append("button")
        .attr("class", "half_line")
		.text("Add Factor Graph")
		.on("click", this.addFactorGraph.bind(this));
}

SelectionFactorApply.prototype.addFactorGraph = function(){
	this.callBack(this.getSelectedFactor());
}

SelectionFactorApply.prototype.getSelectedFactor = function(){
	return this.selectFactor.getSelectedFactor();
}

SelectionFactorApply.prototype.getFactorNumber = function(){
    return Object.keys(this.selectFactor.factorList).length
}

//----------------------------------------------
// Plot Area Selection Object
// ---------------------------------------------

selectionPlotAreaNumber = 0;
SelectionPlotArea = function(exp_id, factor, metricSelectCallBack, parentDomId){
	this.exp_id = exp_id;
	this.factor = factor;
	this.metricSelectCallBack = metricSelectCallBack;
	this.id = "selectionPlotArea_" + selectionPlotAreaNumber;
	selectionPlotAreaNumber += 1;

	this.dSelf = d3.select("#" + parentDomId).append("div")
        .attr("id", this.id)
        .attr("class", "plot_area_selection");
	this.eSelf = document.getElementById(this.id);

    this.levelList = {};
    this.factorList = {};
    this.parameters = {};
    this.selectMetric = new SelectionMetric(this.exp_id, this.metricSelectChange.bind(this), this.id);

    this.selectMetric.readInitialData(function(){
        this.readInitialData(this.initComponents.bind(this));
    }.bind(this));
}

SelectionPlotArea.prototype.readInitialData = function(callBack){
    d3.json("/safe/exp/" + this.exp_id + "/levels/", function(error, data){
        if(error) { console.warn(error) }

        Object.keys(data).forEach(function(factor_id){
            if(factor_id == this.factor.id){ return }
            this.factorList[factor_id] = data[factor_id];
            this.levelList[factor_id] = data[factor_id].levels;
            this.parameters[data[factor_id].name] = data[factor_id].levels[0];
        }.bind(this));
        if(callBack != undefined){ callBack() }
    }.bind(this));
}

SelectionPlotArea.prototype.initComponents = function(){
    var selectors = this.dSelf.selectAll("div").data(Object.keys(this.levelList)).enter()
        .append("div")
        .attr("class", "select_line");

    selectors.append("div")
        .style({"width":"50%", "display":"inline-block"})
        .text(function(d) { return this.factorList[d].name + " : " }.bind(this));

    selectors
        .append("select")
        .attr("factor_id", function(d){ return d })
        .on("change", this.levelSelectChange.bind(this))
        .style({"width":"50%", "display":"inline-block"})
            .selectAll("option").data(function(d){ return this.levelList[d] }.bind(this))
            .enter()
            .append("option")
            .text(function(d){ return d });

    this.selectMetric.initComponents();
    this.selectMetric.setDistinctionId(getKeyString(this.parameters));
}

SelectionPlotArea.prototype.levelSelectChange = function(){
    var select = d3.event.target;
    var level = select.options[select.selectedIndex].value;
    var factor_name = this.factorList[select.getAttribute("factor_id")].name;
    this.parameters[factor_name] = level;
    this.selectMetric.setDistinctionId(getKeyString(this.parameters));
}

SelectionPlotArea.prototype.metricSelectChange = function(metric){
    this.metricSelectCallBack(this.parameters, metric);
}

SelectionPlotArea.prototype.addDisplayed = function(keyString, metric_id, color){
    //console.log("Setting ", metric_id,  " of ", keyString, " to ", color);
    this.selectMetric.setMetricColor(keyString, metric_id, color);
    this.selectMetric.updateSelectedMetrics();
}

SelectionPlotArea.prototype.removeDisplayed = function(keyString, metric_id){
    this.selectMetric.setMetricColor(keyString, metric_id, "white");
    this.selectMetric.updateSelectedMetrics();
}


//----------------------------------------------
// Plot Area Object Object
// ---------------------------------------------

optionsPlotAreaNumber = 0;
OptionsPlotArea = function(exp_id, optionChangeCallBack, parentDomId){
	this.exp_id = exp_id;
	this.optionChangeCallBack = optionChangeCallBack;
	this.id = parentDomId + "_optionsPlotArea_" + optionsPlotAreaNumber;
	selectionPlotAreaNumber += 1;

	this.dSelf = d3.select("#" + parentDomId).append("div")
        .attr("id", this.id)
        .attr("class", "plot_area_options");
	this.eSelf = document.getElementById(this.id);

    //console.log("Options: ", this.dSelf);
    this.initComponents();
}


OptionsPlotArea.prototype.initComponents = function(){
    var id = ["x_axis_label", "y_axis_label"];
    var labelData = ["X Axis Label:", "Y Axis Label:"];

    var labels = this.dSelf.selectAll("div").data(labelData).enter()
        .append("div");

    labels.append("div")
        .attr("class", "select_line")
        .text(function(d) { return d });

    labels.append("input")
        .attr("class", "select_line")
        .attr("id", function(d, e) { return this.id + id[e] }.bind(this)) 
        .on("change", this.labelChange.bind(this));

    this.dSelf.append("div").attr("class", "select_line").text("Move Selected Graph:");
    var position = this.dSelf.append("div").attr("class", "select_line");
    position.append("button").attr("class", "half_line").text("UP").on("click", this.moveSelectedUp.bind(this));
    position.append("button").attr("class", "half_line").text("DOWN").on("click", this.moveSelectedDown.bind(this));

    this.xAxisLabel = document.getElementById(this.id + id[0]);
    this.yAxisLabel = document.getElementById(this.id + id[1]);
}

OptionsPlotArea.prototype.setSelectedGraph = function(graph){
    this.selectedGraph = graph;
    this.setLabels();
}

OptionsPlotArea.prototype.setContextGraph = function(graph){
    this.contextGraph = graph;
}

OptionsPlotArea.prototype.labelChange = function(){
    if(this.selectedGraph){
        this.selectedGraph.setXAxisLabel(this.xAxisLabel.value);
        this.selectedGraph.setYAxisLabel(this.yAxisLabel.value);
    }
    if(this.contextGraph){
        this.contextGraph.setXAxisLabel(this.xAxisLabel.value);
        this.contextGraph.setYAxisLabel(this.yAxisLabel.value);
    }
}

OptionsPlotArea.prototype.setLabels = function(){
    if(this.selectedGraph){
        this.xAxisLabel.value = this.selectedGraph.getXAxisLabel();
        this.yAxisLabel.value = this.selectedGraph.getYAxisLabel();
    }
    if(this.contextGraph){
        this.contextGraph.setXAxisLabel(this.selectedGraph.getXAxisLabel());
        this.contextGraph.setYAxisLabel(this.selectedGraph.getYAxisLabel());
    }
}

OptionsPlotArea.prototype.moveSelectedUp = function(){
    if(!(this.contextGraph && this.selectedGraph)){ return }

    var selected = this.selectedGraph.svg[0][0];
    var context = this.contextGraph.svg[0][0];
    
    selected.parentNode.insertBefore(selected, selected.previousSibling);
    context.parentNode.insertBefore(context, context.previousSibling);
}

OptionsPlotArea.prototype.moveSelectedDown = function(){
    if(!(this.contextGraph && this.selectedGraph)){ return }
 

    var selected = this.selectedGraph.svg[0][0];
    var context = this.contextGraph.svg[0][0];
    
    if(context.nextSibling){ 
        context.parentNode.insertBefore(context, context.nextSibling.nextSibling);
        selected.parentNode.insertBefore(selected, selected.nextSibling.nextSibling);
    }else{
        context.parentNode.insertBefore(context, context.parentNode.firstChild);
        selected.parentNode.insertBefore(selected, selected.parentNode.firstChild);
    }
}


//----------------------------------------------
// Plot Area Object
// ---------------------------------------------

metricPlotAreaNumber = 0;
MetricPlotArea = function(exp_id, factor, parentDomId){
	this.exp_id = exp_id;
    this.factor = factor;
	this.dParent = d3.select("#" + parentDomId);
	this.eParent = document.getElementById(parentDomId);
    this.margin = {top: 10, right: 40, bottom: 50, left: 40};
	this.id = "plotArea_" + metricPlotAreaNumber;
	metricPlotAreaNumber += 1;

	this.colorList = {};		// list of colors by run_id and metric_id
	this.displayList = {};		// list of metrics being displayed by run_id and metric_id
	this.dataList = {};			// list of data that has been loaded by run_id and metric_id
	
    this.graphWidth = undefined;
    this.graphHeight = 180;
	this.plotList = {};
    this.designPointList = {};

    this.selectedGraph = undefined;
    this.contextGraph = undefined;

    d3.json("/safe/exp/" + this.exp_id + "/designpoints/", function(error, designPoints){
        if(error) { console.warn(error) }

        for(var index in designPoints){
            this.designPointList[designPoints[index].id] = designPoints[index];
        }
        this.initComponents();
    }.bind(this)); 
}

MetricPlotArea.prototype.getQueryParameters = function(levelDict){
    var parameters = [];
    //console.log(parameters);
    Object.keys(levelDict).forEach(function(key){
        parameters.push({"name":key , "min_value":levelDict[key] , "max_value":levelDict[key] });
    });
    return parameters;
}

// levelDict -> a dictionary of (level_name, level) pairs
MetricPlotArea.prototype.selectionChange = function(levelDict, metric){
    var metric_id = metric.id;
    var keyString = getKeyString(levelDict);    

    if(this.colorList[keyString] == undefined){ this.colorList[keyString] = {} }
	if(this.displayList[keyString] == undefined) { this.displayList[keyString] = {} }

	if(this.colorList[keyString][metric_id] == undefined) { this.colorList[keyString][metric_id] = colors.shift() }

	if(this.displayList[keyString][metric_id] == undefined){
		this.addDisplayMetric(levelDict, keyString, metric);
	}else{
		this.removeDisplayMetric(keyString, metric.id);
	}
}

MetricPlotArea.prototype.addDisplayMetric = function(levelDict, keyString, metric){
    var metric_id = metric.id;
    var id = keyString + "_" + metric_id;

    var statisticalData = [];
    var designPointIds = [];
    var dataByLevel = {};
    this.displayList[keyString][metric_id] = true;
    this.selection.addDisplayed(keyString, metric_id, this.colorList[keyString][metric_id]);

    // Updates the visual components for the graph being displayed
	var update = function(){
        this.plotList[id].display(id);
        if(this.selectedGraph == undefined){ this.onGraphSelect(this.plotList[id], this.plotList[id].svg[0][0]) }

        var displayString = getDisplayString(levelDict, metric.name);
        var color = this.colorList[keyString][metric_id];
        var returnAttributes = {"key_string": keyString, "metric_id": metric_id };

        this.displayKey.addElement(id, displayString, color, returnAttributes)
        this.displayKey.updateElements();
        this.brushed();
    }.bind(this)

    // Creates a new graph from statisticalData
    var createDisplay = function(){
        //console.log("Statistical Data:", statisticalData);
        this.dataList[keyString][metric_id] = true;

        pointAttributes = {"stroke": this.colorList[keyString][metric_id], 
                           "stroke-width": focusDataPointRadius, 
                           "fill": "white", 
                           "fill-opacity": 0, 
                           "r": focusDataPointRadius};
        
        pointMouseEvent = {"mouseover": mouseOverDataPoint,
                           "mouseout": mouseOutDataPoint };

        pointAttributes["info"] = function(d) { return numberFormat(d.cx)  + "<br/>" + numberFormat(d.cy) };

        this.plotList[id] = new LineMagnitude(this.margin, this.graphWidth, this.graphHeight, this.onGraphSelect.bind(this), this.id + "_graph_area", false);
        this.plotList[id].addDataSet(id, statisticalData, pointAttributes, pointMouseEvent);
        this.plotList[id].setYAxisLabel(metric.name + " ( " + metric.units + " )");
        this.plotList[id].setXAxisLabel(this.factor.name);

        update();
    }.bind(this)

    // Retrieves a list of design points with the given parameters
    var dataFromDesignPoints = function(parameters){
        //console.log("Retreiving design points for: ", parameters);

        d3.json("/safe/exp/" + this.exp_id + "/dp-range-query/").post( parameters, function(error, data){
            if(error){ console.warn(error) }

            designPointIds = data;
            for(var index in designPointIds){
                getDesignPointData(designPointIds[index]);
            }
        }.bind(this));
    }.bind(this)

    // Retreives the simulation runs associated with the specified design point
    var getDesignPointData = function(designPointId){
        //console.log("Retrieving simulation runs for: ", designPointId);
        d3.json("/safe/designpoint/" + designPointId + "/simruns/", function(error, simruns){
            if(error) { console.warn(error) }
            getSimrunData(simruns, designPointId);
        }.bind(this));
    }.bind(this)

    // Retrieves the data for the simulation runs and computes their statistical values
    var getSimrunData = function(simruns, designPointId){
        //console.log("Retrieving results runs for: ", simruns);
        d3.json("/safe/metric/" + metric_id + "/results/").post(JSON.stringify(simruns), function(error, results){
            if(error) { console.warn(error) }

            var level = this.designPointList[designPointId].factors[this.factor.name];
            dataByLevel[level] = results;
            if(Object.keys(dataByLevel).length >= designPointIds.length)
                evaluateStatistics();
        }.bind(this));
    }.bind(this)

    var evaluateStatistics = function(){
        keys = Object.keys(dataByLevel).sort();
        keys.forEach(function(key){
            statisticalData.push(calculateStatistics(key, dataByLevel[key]));
        }.bind(this));

        createDisplay();
    }.bind(this)

    // Determine whether a new graph must be created
    if(this.dataList[keyString] == undefined){ this.dataList[keyString] = {} }
    if(this.dataList[keyString][metric_id] == undefined){
        var parameters = JSON.stringify(this.getQueryParameters(levelDict));
        dataFromDesignPoints(parameters);
    }else{
        this.plotList[id].setVisible(keyString);
		update();
    }

}

MetricPlotArea.prototype.removeMetricFromDisplay = function(attributes){
    //console.log(attributes, attributes.key_string, attributes.metric_id);
    this.removeDisplayMetric(attributes.key_string, attributes.metric_id);
}

MetricPlotArea.prototype.removeDisplayMetric = function(keyString, metric_id){
    var id = keyString + "_" + metric_id;

    this.displayKey.removeElement(id);
    this.selection.removeDisplayed(keyString, metric_id);
    this.plotList[id].svg.remove();

    delete this.dataList[keyString][metric_id];
    delete this.displayList[keyString][metric_id];
    delete this.plotList[id];
}

MetricPlotArea.prototype.initComponents = function(){
    this.dSelf = this.dParent.selectAll("#" + this.id).data([""]).enter()
		.append("div")
        .attr("id", this.id)
        .attr("class", "plot_area");
	this.eSelf = document.getElementById(this.id);
    windowResizeFunctions.push(this.resize.bind(this));

    this.main = this.dSelf.append("div")
        .attr("id", this.id + "_data_area")
        .style({"width":"80%", "display":"inline-block"});
      
    this.main
        .append("div")
        .attr("class", "plot_area_label")
        .text(this.factor.name);

    this.main
      .append("div")
      .attr("height", this.graphHeight + "px")
      .attr("class", "graph_area")
      .attr("id", this.id + "_graph_area");
    this.graphWidth = document.getElementById(this.id + "_graph_area").offsetWidth;

    this.select = this.dSelf.append("div")
        .attr("id", this.id + "_selection_area")
        .style({"width":"20%", "display":"inline-block", "vertical-align":"top"});

    this.select
        .append("div")
        .attr("class", "plot_selection_label")
        .text("Select");
    
	this.selection = new SelectionPlotArea(this.exp_id, this.factor, this.selectionChange.bind(this), this.id + "_selection_area");

    this.select.append("div").style("height","0.5cm");

    this.select
        .append("div")
        .attr("class", "plot_selection_label")
        .text("Options");

    this.options = new OptionsPlotArea(this.exp_id, this.optionChange.bind(this), this.id + "_selection_area");
    this.displayKey = new DisplayElements(this.removeMetricFromDisplay.bind(this) , this.id + "_data_area");
    
    this.context = new LineMagnitude(this.margin, this.graphWidth, this.graphHeight, undefined, this.id + "_graph_area", true);
    this.context.svg.style({"border": "solid 3px red"}); 
    this.options.setContextGraph(this.context);

    this.brush = d3.svg.brush() 
        .x(this.context.dXScale)
        .on("brush", this.brushed.bind(this));

    this.context.svg.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + this.margin.left + "," + this.margin.top + ")")
        .call(this.brush)
        .selectAll("rect")
            .attr("y", -6)
            .attr("height", this.context.height + 7);
}

MetricPlotArea.prototype.optionChange = function(){
}

MetricPlotArea.prototype.onGraphSelect = function(graph, src){
    if(this.selectedGraph)
        this.selectedGraph.svg.style("border", "none");
    graph.svg.style({"border": "solid 1px red"});

    this.selectedGraph = graph;
    this.context.removeAll();

    var id = graph.getVisibleIds()[0];
    var dataSet = graph.getDataSet(id);
    var pointAttributes = graph.getPointAttributes(id);
    var mouseEvents = graph.getMouseEvents(id);
    
    //console.log( "Extracted: ", id, dataSet, pointAttributes, mouseEvents);
    this.context.addDataSet(id, dataSet, pointAttributes, mouseEvents);
    this.context.display(id);

    this.options.setSelectedGraph(graph);
    if(src == undefined){ src = d3.event.srcElement }
    var cxt = this.context.svg[0][0];
    cxt.parentNode.insertBefore(cxt, src.nextSibling);
}

MetricPlotArea.prototype.brushed = function() {
    var extent = this.brush.extent();
    xDomain = this.brush.empty() ? this.context.dXScale.domain() : [ extent[0], extent [1] ];

    Object.keys(this.plotList).forEach(function(key){
        this.plotList[key].updateDomains(xDomain);selected = 
        this.plotList[key].displayAll();
    }.bind(this));
}

MetricPlotArea.prototype.resize = function(){
    console.log("Resizing...");
    var graphArea = document.getElementById(this.id + "_graph_area");
    if(graphArea == undefined){ return }

    this.graphWidth = graphArea.offsetWidth;
    this.context.setSize(this.graphWidth, this.graphHeight);
    this.context.displayAll();
    this.context.svg.selectAll(".brush")
        .attr("transform", "translate(" + this.margin.left + "," + this.margin.top + ")")
        .call(this.brush)

    Object.keys(this.plotList).forEach(function(key){
        this.plotList[key].setSize(this.graphWidth, this.graphHeight);
        this.plotList[key].displayAll();
    }.bind(this));
}

//======================================================

function onWindowResize(){
    console.log(windowResizeFunctions);
    windowResizeFunctions.forEach(function(resizeFunction){
        resizeFunction();
    });
}

function initVisualComponents(){
    d3.select(window).on("resize", onWindowResize);
	selectFactor = new SelectionFactorApply(exp_id, addFactorPlotArea, "select_factor");
}

function addFactorPlotArea(factor){
	//console.log("Factor Changed: ", factor);
	new MetricPlotArea(exp_id, factor, "place_graphs");
    d3.select("#place_graphs").append("div").attr("class", "graph_sep");
}

function getKeyString(levelDict){
    var keyString = "";
    Object.keys(levelDict).sort().forEach(function(key){
        keyString += key + "_" + levelDict[key] + "_";
    });
    keyString = keyString.split(".").join("_");
    return keyString;
}

function getDisplayString(levelDict, metric_name){
    var displayString = metric_name;
    Object.keys(levelDict).sort().forEach(function(key){
        displayString += ", " + key + " = " + levelDict[key]
    });
    return displayString;
}


// called when the mouse is positioned over a data element
function mouseOverDataPoint(){
    console.log("Data:", d3.event.pageX, d3.event.pageY);
	infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);

    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}

function brushed() {
    var extent = brush.extent();
    xDomain = brush.empty() ? context.dXScale.domain() : [ extent[0][0], extent [1][0] ];
    yDomain = brush.empty() ? context.dYScale.domain() : [ extent[0][1], extent [1][1] ];

   focus.updateDomains(xDomain, yDomain); 
   focus.displayAll();
}

function enableCrossHairs(){
    if(this.checked){
        focus.enableCrossHairs();
    }else{
        focus.disableCrossHairs()
    }
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
}

function calculateStatistics(level, data){
    //console.log(data);
    data.sort(function(a, b) { return a.value - b.value});
    data.forEach(function(d, i) { data[i] = d.value });
    statistics = {};
    
    statistics["xValue"] = parseFloat(level);
    statistics["minValue"] = d3.quantile(data, 0);
    statistics["lowDevValue"] = d3.quantile(data, 0.25);
    statistics["medianValue"] = d3.quantile(data, 0.50);
    statistics["highDevValue"] = d3.quantile(data, 0.75);
    statistics["maxValue"] = d3.quantile(data, 1);

    return statistics;
}

readParameters();
initVisualComponents();
