var safe = {};

(function(){
    this.getD3Object = function(element){
        if(element.tagName){
            return d3.select(element);
        }else if(typeof(element) == "string" || element instanceof String){
            return d3.select("#" + element);
        }else{
            return element;
        }
    }

    //----------------------------------------------
    // this.full  Object Prototypes
    // ---------------------------------------------

    // Initializes the selection for the graph
    // callBack of the form callBack(run_id, metric_id)
    this.selectors = {};
    this.selectors.full = function(){
        this.init = function(exp_id, selectionCallBack, applyFilterCallBack, parentElement){
            this.exp_id = exp_id;
            this.dParent = safe.getD3Object(parentElement);
            this.selectionCallBack = selectionCallBack;
            this.applyFilterCallBack = applyFilterCallBack;
            
            this.dSelf = this.dParent.append("div");

            this.selectFactor = new safe.selectors.factor(this.exp_id, this.factorSelectChange.bind(this), this.dSelf);
            this.selectLevelMin = new safe.selectors.level(this.exp_id, undefined, this.dSelf);
            this.selectLevelMax = new safe.selectors.level(this.exp_id, undefined, this.dSelf);
            this.selectDesignPoint = new safe.selectors.designpoint(this.exp_id, this.designPointSelectChange.bind(this), this.dSelf);
            this.selectRun = new safe.selectors.run(this.exp_id, this.runSelectChange.bind(this), this.dSelf);
            this.selectMetric = new safe.selectors.metric(this.exp_id, this.selectElementChange.bind(this), this.dSelf);

            this.selectFactor.readInitialData(function(){
                this.selectLevelMin.readInitialData(function(){
                    this.selectLevelMax.readInitialData(function(){
                        this.selectMetric.readInitialData(this.initComponents.bind(this))
                    }.bind(this));
                }.bind(this));
            }.bind(this));
        }
            
        this.initComponents = function(){
            // Initialize the factor list
            this.selectFactor.initComponents();
            this.selectLevelMin.initComponents();
            this.selectLevelMax.initComponents();
             
            // Initialize the apply selection button
            this.dSelf
                .append("button")
                .attr("class", "apply_selection")
                .text("Apply Selection Filter")
                .on("click", this.applyDesignFilter.bind(this));

            this.selectDesignPoint.initComponents();
            this.selectRun.initComponents();
            this.selectMetric.initComponents();
              
            this.factorSelectChange();
            this.updateDesignPoints();
        }

        this.factorSelectChange = function(){

            var factor_id = this.selectFactor.getSelectedFactor().id;

            this.selectLevelMin.updateLevels(factor_id);
            this.selectLevelMax.updateLevels(factor_id);

            this.selectLevelMax.eSelect.selectedIndex = this.selectLevelMax.eSelect.options.length - 1;
        }

        this.applyDesignFilter = function(){

            var factor = this.selectFactor.getSelectedFactor();
            var min = this.selectLevelMin.getMinLevel(factor.id);
            var max = this.selectLevelMax.getMaxLevel(factor.id);

            if(min > max){
                alert("The factor's beginning level cannot be greater than the factor's ending level");
                return;
            }

            var parameter = {"name":factor.name, "factor_id":factor.id, "min_value":this.getMinLevel(), "max_value":this.getMaxLevel() };
            if(this.getMinLevel() > min || this.getMaxLevel() < max ){
                this.selectDesignPoint.addParameter(factor.id, parameter);
            }else{
                this.removeDesignFilter(factor.id);
            }

            this.updateDesignPoints();
            this.applyFilterCallBack(parameter);
        }

        this.removeDesignFilter = function(factor_id){
            this.selectDesignPoint.removeSelectionParameter(factor_id);

            this.updateDesignPoints();
        }

        this.updateDesignPoints = function(){
            this.selectDesignPoint.updateDesignPoints(this.designPointSelectChange.bind(this));
        }

        this.designPointSelectChange = function(){
            this.selectRun.updateRuns(this.selectDesignPoint.getSelectedDesignPoint(), function(){
                this.selectMetric.setDistinctionId(this.getSelectedRun());
                this.selectMetric.updateSelectedMetrics();  
            }.bind(this));
        }

        this.runSelectChange = function(){
            this.selectMetric.setDistinctionId(this.getSelectedRun());
            this.selectMetric.updateSelectedMetrics();
        }

        this.addDisplayed = function(run_id, metric_id, color){
            this.selectMetric.setMetricColor(run_id, metric_id, color);
            this.selectMetric.updateSelectedMetrics();
        }

        this.getDesignParameters = function(){
            return this.selectDesignPoint.getSelectionParameters();
        }

        this.removeDisplayed = function(run_id, metric_id){
            this.selectMetric.setMetricColor(run_id, metric_id, "white");
            this.selectMetric.updateSelectedMetrics();
        }

        this.selectElementChange = function(){
            var metric_id = d3.event.toElement.getAttribute("metric_id");
            var metric = this.selectMetric.getMetric(metric_id);
            if(this.selectionCallBack){ this.selectionCallBack(this.getSelectedRun(), metric) }
        }

        this.getSelectedFactor = function(){
            return this.selectFactor.getSelectedFactor();
        }

        this.getMinLevel = function(){
            return this.selectLevelMin.getSelectedLevel();
        }

        this.getMaxLevel = function(){
            return this.selectLevelMax.getSelectedLevel();
        }

        this.getSelectedDesignPoint = function(){
            return this.selectDesignPoint.getSelectedDesignPoint();
        }

        this.getSelectedRun = function(){
            return this.selectRun.getSelectedRun();
        }
    }


    //----------------------------------------------
    // Factor Selection Object Prototypes
    // ---------------------------------------------

    this.selectors.factorInstances = 0;
    this.selectors.factor = function(exp_id, changeCallBack, parentElement){
        this.exp_id = exp_id;
        this.callBack = changeCallBack;
        this.dParent = safe.getD3Object(parentElement);
        this.factorList = {};

        this.readInitialData = function(callBack){
            d3.json("/safe/exp/" + exp_id + "/factors/", function(error, data){
                if(error){ console.warn(error) }

                data.forEach(function(factor){ this.factorList[factor.id] = factor }.bind(this));
                if(callBack != undefined){ callBack() }
            }.bind(this));
        }

        this.initComponents = function(attributes, style){
            this.dSelect = this.dParent
                .append("select")
                .attr("class", "factor_select")
                .style(style)
                .on("change", this.selectChange.bind(this));
            this.eSelect = this.dSelect[0][0];

            this.dSelect
                .selectAll("option")
                .data(Object.keys(this.factorList))
                .enter()
                .append("option")
                .attr("factor", function(d) { return d })
                .text(function(d) { return this.factorList[d].name }.bind(this));
        }

        this.selectChange = function(){
            if(this.callBack != undefined){ this.callBack(this.getSelectedFactor()) }
        }

        this.getSelectedFactor = function(){
            var factor_id = this.eSelect.options[this.eSelect.selectedIndex].getAttribute("factor");
            return this.factorList[factor_id];
        }
    }

    //----------------------------------------------
    // Level Selection Object Prototypes
    // ---------------------------------------------

    this.selectors.level = function(exp_id, changeCallBack, parentElement){
        this.exp_id = exp_id;
        this.callBack = changeCallBack;
        this.dParent = safe.getD3Object(parentElement);
        this.levelList = {};

        this.readInitialData = function(callBack){
            d3.json("/safe/exp/" + exp_id + "/levels/", function(error, data){
                if(error){ console.warn(error) }

                Object.keys(data).forEach(function(factor_id){ 
                    this.levelList[factor_id] = data[factor_id].levels.sort(); 
                }.bind(this));

                if(callBack != undefined){ callBack() }
            }.bind(this));
        }

        this.initComponents = function(attributes, styles){
            this.dSelect = this.dParent
                .append("select")
                .attr(attributes)
                .attr("class", "level_select")
                .style(styles)
                .on("change", this.selectChange.bind(this));
            this.eSelect = this.dSelect[0][0];
        }

        this.updateLevels = function(factor_id){
            levels = this.dSelect.selectAll("option").data(this.levelList[factor_id])
            levels.enter().append("option");
            levels.attr("level", function(d) { return d }).text(function(d) { return d });
            levels.exit().remove();
            this.eSelect.selectedIndex = 0;
        }

        this.setLevels = function(factor_id, levels){
            this.levelList[factor_id] = levels;
        }

        this.selectChange = function(){
            if(this.callBack){ this.callBack(this.getSelectedLevel()) }
        }

        this.getSelectedLevel= function(){
            return this.eSelect.options[this.eSelect.selectedIndex].getAttribute("level");
        }

        this.getMinLevel = function(factor_id){
            return this.levelList[factor_id][0];
        }

        this.getMaxLevel = function(factor_id){
            return this.levelList[factor_id][this.levelList[factor_id].length - 1 ];
        }
    }

    //----------------------------------------------
    // Design Point Selection Object Prototypes
    // ---------------------------------------------

    this.selectors.designpoint = function(exp_id, changeCallBack, parentElement){
        this.exp_id = exp_id;
        this.callBack = changeCallBack;
        this.dParent = safe.getD3Object(parentElement);

        this.designPointList = {};
        this.designPointFactors = undefined;
        this.parameters = {};

        this.readInitialData = function(callBack){
            this.designPointFactors = {};
            d3.json("/safe/exp/" + this.exp_id + "/designpoints/", function(error, designPoints){
                designPoints.forEach(function(designPoint){
                    this.designPointFactors[designPoint.id] = designPoint.factors;
                }.bind(this));
                if(callBack != undefined){ callBack() }
            }.bind(this));
        }

        this.readData = function(callBack){
            var update = function(){ 
                var parameters = this.getSelectionParameters();

                d3.json("/safe/exp/" + this.exp_id + "/dp-range-query/").post(JSON.stringify(parameters), function(error, designPoints){
                    this.designPointList = designPoints;
                    if(callBack != undefined){ callBack() };
                }.bind(this));
            }.bind(this)
            
            if(this.designPointFactors == undefined ){
                this.readInitialData(update.bind(this));
            }else{
              update();
            }
        }

        this.initComponents = function(attributes){
            this.dSelf = this.dParent
                .append("div");

            this.dSelect = this.dSelf
                .append("select")
                .attr(attributes)
                .attr("class", "designpoint_select")
                .on("change", this.selectChange.bind(this));
            this.eSelect = this.dSelect[0][0];

            this.dInfo = this.dSelf
                .append("div")
                .style("outline", "solid 1px black");
        }

        this.updateDesignPoints = function(callBack){
            var update = function(){
                designPoints = this.dSelect.selectAll("option").data(this.designPointList);
                designPoints.enter().append("option");
                designPoints.attr("designPoint", function(d) { return d })
                    .text(function(d) { return "DesignPoint " + d });
                designPoints.exit().remove();

                this.updateDesignPointInfo();
                
                if(callBack != undefined){ callBack() }
            }.bind(this);
            this.readData(update.bind(this));
        }

        this.updateDesignPointInfo = function(){
            var designPoint = this.designPointFactors[this.getSelectedDesignPoint()];

            var lines = this.dInfo.selectAll(".key_line").data(Object.keys(designPoint))
            lines.enter().append("div");
            lines.attr("class", "key_line").text(function(d) { return d + " : " + designPoint[d] });
            lines.exit().remove();
            lines.sort();
        }

        this.selectChange = function(){
            this.updateDesignPointInfo();
            this.callBack(this.getSelectedDesignPoint());
        }

        this.getSelectedDesignPoint = function(){
            designpoint_id = this.eSelect.options[this.eSelect.selectedIndex].getAttribute("designpoint");
            return designpoint_id;
        }

        this.addSelectionParameter = function(key, parameter){
            this.parameters[key] = parameter;
        }

        this.removeSelectionParameter = function(key){
            delete this.parameters[key];
        }

        this.getSelectionParameters = function(){
            parameters = [];
            Object.keys(this.parameters).forEach(function(parameterKey){
                parameters.push(this.parameters[parameterKey]);
            }.bind(this));
            return parameters;
        }
    }

    //----------------------------------------------
    // Run  Object Prototypes
    // ---------------------------------------------

    this.selectors.run = function(exp_id, changeCallBack, parentElement){
        this.exp_id = exp_id;
        this.callBack = changeCallBack;
        this.dParent = safe.getD3Object(parentElement);
        this.runsByDesignPoint = {};

        this.readData = function(designPoint_id, callBack){
            d3.json("/safe/designpoint/" + designPoint_id + "/simruns/", function(error, data){
                if(error) { console.warn(error) }

                this.runsByDesignPoint[designPoint_id] = data;
                callBack();
            }.bind(this));
        }

        this.initComponents = function(attributes){
            this.dSelect = this.dParent
                .append("select")
                .attr(attributes)
                .attr("class", "run_select")
                .on("change", this.selectChange.bind(this));
            this.eSelect = this.dSelect[0][0];
        }

        this.updateRuns = function(designPoint_id, callBack){
            var update = function(){
                runs = this.dSelect.selectAll("option").data(this.runsByDesignPoint[designPoint_id]);
                runs.enter().append("option");
                runs.attr("run", function(d) { return d.simrun_id }).text(function(d) { return "Run " + d.simrun_id });
                runs.exit().remove();
                callBack();
            }.bind(this);
            
            if(this.runsByDesignPoint[designPoint_id] == undefined){
                this.readData(designPoint_id, update);
            }else{
                update();
            }
        }

        this.selectChange = function(){
            this.callBack(this.getSelectedRun());
        }

        this.getSelectedRun = function(){
            return this.eSelect.options[this.eSelect.selectedIndex].getAttribute("run");
        }
    }

    //----------------------------------------------
    // Metric Selection Object Prototypes
    // ---------------------------------------------

    this.selectors.metric = function(exp_id, changeCallBack, parentElement){
        this.exp_id = exp_id;
        this.callBack = changeCallBack;
        this.dParent = safe.getD3Object(parentElement);
        this.metricList = {};
        this.colorList = {};
        this.distinctionId = undefined;

        this.readInitialData = function(callBack){
            d3.json("/safe/exp/" + this.exp_id + "/metrics/", function(error, data){
                if(error) { console.warn(error) }

                data.forEach(function(metric){
                    this.metricList[metric.id] = metric;
                }.bind(this));
                callBack();
            }.bind(this));
        }

        this.initComponents = function(){
            this.dSelect = this.dParent
                .append("div")
                .style("outline", "solid 1px black");
            this.eSelect = this.dSelect[0][0];

            select_items = this.dSelect.selectAll(".key_line").data(Object.keys(this.metricList)).enter()
                .append("div")
                .attr("class", "select_metric_line")
                .attr("metric_id", function(d) { return this.metricList[d].id }.bind(this))
                .on("click", this.selectChange.bind(this));
                

            select_items.append("div")
                .attr("class", function(d) { return "select_metric_square select_metric_square_" + d }.bind(this))
                .attr("metric_id", function(d) { return d }.bind(this));

            select_items.append("div")
                .attr("class", "select_metric_label")
                .attr("metric_id", function(d) {  return d }.bind(this))
                .text(function(d) { return this.metricList[d].name}.bind(this));

        }

        this.setMetricColor = function(distinctionId, metric_id, color){
            if(this.colorList[distinctionId] == undefined) { this.colorList[distinctionId] = {} }
            this.colorList[distinctionId][metric_id] = color;
            this.updateSelectedMetrics();
        }

        this.updateSelectedMetrics = function(){
            Object.keys(this.metricList).forEach(function(metric_id){
                if(this.colorList[this.distinctionId] != undefined && this.colorList[this.distinctionId][metric_id] != undefined){
                    this.dParent.selectAll(".select_metric_square_" + metric_id)
                        .style("background", this.colorList[this.distinctionId][metric_id]);
                }else{
                    this.dParent.selectAll(".select_metric_square_" + metric_id)
                        .style("background", "white");
                }
            }.bind(this));
        }

        this.setDistinctionId = function(distinctionId){
            this.distinctionId = distinctionId;
            this.updateSelectedMetrics();
        }

        this.selectChange = function(){
            var metric_id = d3.event.toElement.getAttribute("metric_id");
            this.callBack(this.metricList[metric_id]);
        }

        this.getMetric = function(metric_id){
            return this.metricList[metric_id];
        }
    }

    //----------------------------------------------
    // Filter selection Object Prototypes
    // ---------------------------------------------

    this.selectors.results = function(parentElement){
        this.exp_id = exp_id;
        this.dParent = safe.getD3Object(parentElement);

        this.initComponents = function(){

            this.dSelect = this.dParent
                .append("div")
                .style("outline", "solid 1px black");
            this.eSelect = this.dSelect[0][0];

            var data = [{name: "Filter Step:"},
                        {name: "Max Results:"},
                        {name: "Low Pass Filter"}];

            var lines = this.dSelect.selectAll(".key_line").data(data);

            lines = lines.enter().append("div").attr("class", "key_line");

            lines
                .append("div")
                .attr("class", "half_line")
                .text(function(d) { return d.name });

            var inputs = lines
                .append("input")
                .attr("type", "number")
                .attr("min", 0)
                .attr("value", 0)
                .attr("class", "bordered_half_line");

            this.clusterSize = inputs[0][0];
            this.maxResults = inputs[0][1];
            this.denoiseWindowSize = inputs[0][2];
        }

        this.setMetricColor = function(distinctionId, metric_id, color){
            if(this.colorList[distinctionId] == undefined) { this.colorList[distinctionId] = {} }
            this.colorList[distinctionId][metric_id] = color;
            this.updateSelectedMetrics();
        }

        this.getClusterSize = function(){
            if (this.clusterSize.value == ""){ this.clusterSize.value = 0 }
            return this.clusterSize.value;
        }

        this.getMaxResults = function(){
            if (this.maxResults.value == ""){ this.maxResults.value = 0 }
            return this.maxResults.value;
        }

        this.getDenoiseWindowSize = function(){
            if (this.denoiseWindowSize.value == ""){ this.denoiseWindowSize.value = 0 }
            return this.denoiseWindowSize.value;
        }

        this.getResultsParameters = function(){
            return {"filter_step": this.getClusterSize(),
                    "max_results":this.getMaxResults(),
                    "denoise_window":this.getDenoiseWindowSize()};
        }
    }


    //----------------------------------------------
    // Options for a selected graph 
    // ---------------------------------------------

    this.selectors.graph = function(exp_id, optionChangeCallBack, parentElement){
        this.exp_id = exp_id;
        this.optionChangeCallBack = optionChangeCallBack;

        this.dSelf = safe.getD3Object(parentElement).append("div")
            .attr("class", "plot_area_options");
        this.eSelf = this.dSelf[0][0];

        this.initComponents = function(){
            var labelData = ["X Axis Label:", "Y Axis Label:"];

            var labels = this.dSelf.selectAll("div").data(labelData).enter()
                .append("div");

            labels.append("div")
                .attr("class", "select_line")
                .text(function(d) { return d });

            var inputs = labels.append("input")
                .attr("class", "select_line")
                .on("change", this.labelChange.bind(this));

            this.dSelf.append("div").attr("class", "select_line").text("Move Selected Graph:");
            var position = this.dSelf.append("div").attr("class", "select_line");
            position.append("button").attr("class", "half_line").text("UP").on("click", this.moveSelectedUp.bind(this));
            position.append("button").attr("class", "half_line").text("DOWN").on("click", this.moveSelectedDown.bind(this));

            this.xAxisLabel = inputs[0][0];
            this.yAxisLabel = inputs[0][1];
        }

        this.setSelectedGraph = function(graph){
            this.selectedGraph = graph;
            this.setLabels();
        }

        this.setContextGraph = function(graph){
            this.contextGraph = graph;
        }

        this.labelChange = function(){
            if(this.selectedGraph){
                this.selectedGraph.setXAxisLabel(this.xAxisLabel.value);
                this.selectedGraph.setYAxisLabel(this.yAxisLabel.value);
            }
            if(this.contextGraph){
                this.contextGraph.setXAxisLabel(this.xAxisLabel.value);
                this.contextGraph.setYAxisLabel(this.yAxisLabel.value);
            }
        }

        this.setLabels = function(){
            if(this.selectedGraph){
                this.xAxisLabel.value = this.selectedGraph.getXAxisLabel();
                this.yAxisLabel.value = this.selectedGraph.getYAxisLabel();
            }
            if(this.contextGraph){
                this.contextGraph.setXAxisLabel(this.selectedGraph.getXAxisLabel());
                this.contextGraph.setYAxisLabel(this.selectedGraph.getYAxisLabel());
            }
        }

        this.moveSelectedUp = function(){
            if(!(this.contextGraph && this.selectedGraph)){ return }

            var selected = this.selectedGraph.svg[0][0];
            var context = this.contextGraph.svg[0][0];
            
            selected.parentNode.insertBefore(selected, selected.previousSibling);
            context.parentNode.insertBefore(context, context.previousSibling);
        }

        this.moveSelectedDown = function(){
            if(!(this.contextGraph && this.selectedGraph)){ return }
         

            var selected = this.selectedGraph.svg[0][0];
            var context = this.contextGraph.svg[0][0];
            
            if(context.nextSibling){ 
                context.parentNode.insertBefore(context, context.nextSibling.nextSibling);
                selected.parentNode.insertBefore(selected, selected.nextSibling.nextSibling);
            }else{
                context.parentNode.insertBefore(context, context.parentNode.firstChild);
                selected.parentNode.insertBefore(selected, selected.parentNode.firstChild);
            }
        }

        this.initComponents();
    }


    this.graphs = {};
    //--------------------------------
    // Base Magnitude
    // -------------------------------
    this.graphs.base = function(margins, width, height, selectedCallBack, parentElement){
        this.xMin = this.yMin = this.xMax = this.yMax = 0;
        this.data = {};
        this.toDisplay = {};
        this.pointAttributes = {};
        this.mouseEvents = {};
        this.jMargin = margins;
        this.height = height - this.jMargin.top - this.jMargin.bottom;
        this.width = width - this.jMargin.left - this.jMargin.right;
        this.selectedCallBack = selectedCallBack;
        console.log("Call Back:", selectedCallBack);

        this.dXScale = d3.scale.linear().range([0, this.width]).domain([0, 0]);
        this.dYScale = d3.scale.linear().range([this.height, 0]).domain([0, 0]);

        this.dXAxis = d3.svg.axis().scale(this.dXScale).orient("bottom");
        this.dYAxis = d3.svg.axis().scale(this.dYScale).orient("left");

        this.svg = d3.select("#" + parentElement)
            .append("svg")
            .attr("width", this.width + this.jMargin.left + this.jMargin.right)
            .attr("height", this.height + this.jMargin.top + this.jMargin.bottom)
            .on("click", this.selected.bind(this));
        console.log("SVG:", this.svg);

        // Text label for x axis
        this.xAxisLabel = this.svg.append("text")
            .attr("class", "x label")
            .attr("x", this.width / 2)
            .attr("y", this.height + this.jMargin.top)
            .attr("dy", "3em")
            .style("text-anchor", "middle")
            .text("Original x-axis");

        // Add text label for y axis
        this.yAxisLabel = this.svg.append("text")
            .attr("class", "y label")
            .attr("transform", "rotate(-90)")
            .attr("y", 0)
            .attr("x", 0 - ((this.height) / 2))
            .attr("dy", "1em")
            .style("text-anchor", "middle")
            .text("Original y-axis");

        // Add x axis
        this.svg.append("g")
            .attr("class", "x axis")
            .attr("transform", "translate(" + this.jMargin.left + "," + (this.height + this.jMargin.top) + ")")
            //.on("click", this.selected.bind(this))
            .call(this.dXAxis);

        // Add y axis
        this.svg.append("g")
            .attr("class", "y axis")
            .attr("transform", "translate(" + this.jMargin.left + "," + this.jMargin.top + ")")
            .call(this.dYAxis);

        // Set the cliping area
        this.clip = this.svg.append("defs").append("clipPath")
            .append("rect")
            .attr("class", "clip-rect")
            .attr("y", this.jMargin.top)
            .attr("x", this.jMargin.left)
            .attr("width", this.width)
            .attr("height", this.height);

        return this;
    }

    this.graphs.base.prototype.selected = function(){
        console.log("Selected:", this);
        if(this.selectedCallBack){ this.selectedCallBack(this) }
    }

    this.graphs.base.prototype.setXAxisLabel = function(label){
        this.xAxisString = label;
        this.svg.selectAll(".x.label").text(label);
    }

    this.graphs.base.prototype.setYAxisLabel = function(label){
        this.yAxisString = label;
        this.svg.selectAll(".y.label").text(label);
    }

    this.graphs.base.prototype.getXAxisLabel = function(){
        return this.xAxisString;
    }

    this.graphs.base.prototype.getYAxisLabel = function(){
        return this.yAxisString;
    }

    this.graphs.base.prototype.getWidth = function(){
        return this.width + this.jMargin.left + this.jMargin.right;
    }

    this.graphs.base.prototype.getHeight = function(){
        return this.height + this.jMargin.top + this.jMargin.bottom;
    }

    this.graphs.base.prototype.displayAll = function(){
        Object.keys(this.data).forEach(function(id){
            if(this.toDisplay[id] == true) { this.display(id) }
        }.bind(this));
    }

    this.graphs.base.prototype.addDataSet = function(id, data, pointAttributes, mouseEvents){
        this.data[id] = data;
        this.pointAttributes[id] = pointAttributes;
        this.mouseEvents[id] = mouseEvents;
        this.toDisplay[id] = true;
        this.updateDataFeatures();
    }

    this.graphs.base.prototype.getDataSet = function(id){
        return this.data[id];
    }

    this.graphs.base.prototype.getPointAttributes = function(id){
        return this.pointAttributes[id];
    }

    this.graphs.base.prototype.getMouseEvents = function(id){
        return this.mouseEvents[id];
    }

    this.graphs.base.prototype.setInvisible = function(id){
        this.toDisplay[id] = false;
        this.updateDataFeatures();
        this.display(id);
    }

    this.graphs.base.prototype.setVisible = function(id){
        this.toDisplay[id] = true;
        this.updateDataFeatures();
        this.display(id);
    }

    this.graphs.base.prototype.getVisibleIds = function(){
        values = [];
        Object.keys(this.toDisplay).forEach(function(id){
            if(this.toDisplay[id] == true)
                values.push(id);
        }.bind(this));
        return values;
    }

    this.graphs.base.prototype.remove = function(id){
        delete this.data[id];
        delete this.pointAttributes[id];
        delete this.mouseEvents[id];
    }

    this.graphs.base.prototype.removeAll = function(){
        Object.keys(this.data).forEach(function(id){
            this.setInvisible(id);
            this.remove(id);
        }.bind(this));
    }

    this.graphs.base.prototype.hasData = function(id){
        if( this.data[id] != undefined ){ return true }
        else { return false }
    }

    this.graphs.base.prototype.setHeight = function(height){
        this.setSize(this.getWidth(), height);
    }

    this.graphs.base.prototype.setWidth = function(width){
        this.setSize(width, this.getHeight());
    }

    this.graphs.base.prototype.setSize = function(width, height){
        this.svg
            .attr("width", width)
            .attr("height", height);

        this.width = width - this.jMargin.left - this.jMargin.right;
        this.height = height - this.jMargin.top - this.jMargin.bottom;

        this.xAxisLabel.attr({"x": this.width / 2, "y": this.height + this.jMargin.top } );
        this.yAxisLabel.attr("x", 0-((this.height) / 2));


        this.svg.select(".x.axis").attr("transform", "translate(" + this.jMargin.left + "," + (this.height + this.jMargin.top) + ")");

        this.dYScale.range([this.height, 0]);
        this.dXScale.range([0, this.width]);

        this.svg.select(".x.axis").call(this.dXAxis);
        this.svg.select(".y.axis").call(this.dYAxis);

        this.clip.select(".clip-rect")
            .attr("x", this.jMargin.left)
            .attr("y", this.jMargin.top)
            .attr("width", this.width)
            .attr("height", this.height);
    }

    this.graphs.base.prototype.updateDomains = function(xDomain, yDomain){
        if(xDomain != undefined) { this.dXScale.domain(xDomain) }
        if(yDomain != undefined) { this.dYScale.domain(yDomain) }

        this.svg.select(".x.axis").call(this.dXAxis);
        this.svg.select(".y.axis").call(this.dYAxis);
    }

    this.graphs.base.prototype.updateDataFeatures = function(){
        this.xMin = this.xMax = this.yMin = this.yMax = undefined;
        Object.keys(this.data).forEach(function(id){
            if(this.toDisplay[id] != true){ return }

            xRange = d3.extent(this.data[id], function(d) { return d.time });
            this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
            this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

            yRange = d3.extent(this.data[id], function(d) { return d.value});
            this.yMin = (this.yMin == undefined) ? yRange[0] : d3.min([this.yMin, yRange[0]]);
            this.yMax = (this.yMax == undefined) ? yRange[1] : d3.max([this.yMax, yRange[1]]);
            }.bind(this));

        console.log(this.xMin, this.xMax);
        this.updateDomains([this.xMin, this.xMax], [this.yMin, this.yMax]);
    }

    this.graphs.base.prototype.updateCrossHairs = function(){
        var x = d3.event.offsetX;
        var y = d3.event.offsetY;

        var xleft = this.dXScale.range()[0] + this.jMargin.left; 
        var xright = this.dXScale.range()[1] + this.jMargin.left;
        var ytop = this.dYScale.range()[0] + this.jMargin.top;
        var ybottom = this.dYScale.range()[1] + this.jMargin.top;

        if (x < xleft) { x = xleft }
        if (x > xright) { x = xright }
        if (y < ybottom) { y = ybottom }
        if (y > ytop) { y = ytop }

        horizontal = this.svg.selectAll(".crosshairhorizontal").data([""]);
        vertical = this.svg.selectAll(".crosshairvertical").data([""]);
     
        horizontal.enter().append("line").attr("class", "crosshairhorizontal").style("stroke", "black").style("pointer-events", "none")
        vertical.enter().append("line").attr("class", "crosshairvertical").style("stroke", "black").style("pointer-events", "none")
       
        horizontal.attr("x1", xleft).attr("x2", xright).attr("y1", y).attr("y2", y);
        vertical.attr("y1", ybottom).attr("y2", ytop).attr("x1", x).attr("x2", x);

        horizontal.exit().remove();
        vertical.exit().remove();
    }

    this.graphs.base.prototype.enableCrossHairs = function(){
        this.svg.on("mousemove", this.updateCrossHairs.bind(this));
        this.svg.on("mouseout", function(){
            this.svg.selectAll(".crosshairhorizontal").remove();
            this.svg.selectAll(".crosshairvertical").remove();
            }.bind(this));
    }

    this.graphs.base.prototype.disableCrossHairs = function(){
        this.svg.selectAll(".crosshairhorizontal").remove();
        this.svg.selectAll(".crosshairvertical").remove();
        this.svg.on("mousemove", undefined);
    }


    //--------------------------------
    // Scatterplot Magnitude
    // -------------------------------

    this.graphs.scatterplot = function(margins, width, height, onSelectedCallBack, parentElement){
        this.init = function(margins, width, height, onSelectedCallBack, parentElement){
            BaseMagnitude.apply(this);
            this.init(margins, width, height, onSelectedCallBack, parentElement); 

            return this;
        }

        ScatterMagnitude.prototype.display = function(id){
            if(this.toDisplay[id] != true) { 
                this.svg.selectAll(".dataPoint_" + id).remove();
                return;
            }

            //currentData = [];
            //this.data[id].forEach(function(dataPoint){
            //    if(this.dXScale.domain()[0] <= dataPoint.time && dataPoint.time <= this.dXScale.domain()[1])
            //        if(this.dYScale.domain()[0] <= dataPoint.value && dataPoint.value <= this.dYScale.domain()[1])
            //            currentData.push(dataPoint);
            //    }.bind(this));

            points = this.svg.selectAll(".dataPoint_" + id).data(this.data[id]);
            points.enter()
                .append("circle")
                .attr("clip-path", "url(#" + this.id + "_clip)" )
                .attr("class", "dataPoint_" + id)
                .on(this.mouseEvents[id]); 

            points
                .attr("cx", function (d) { return this.jMargin.left + this.dXScale(d.time) }.bind(this))
                .attr("cy", function (d) { return this.jMargin.top + this.dYScale(d.value) }.bind(this))
                .attr(this.pointAttributes[id]);

            points.exit().remove();	
        }

        this.init(margins, width, height, onSelectedCallBack, parentElement);
    }


    //--------------------------------
    // Lineplot Magnitude
    // -------------------------------


    this.graphs.lineplot = function(margins, width, height, onSelectedCallBack, parentElement, pad){
        //console.log("Initializing LineMagnitude...");
        BaseMagnitude.apply(this);
        this.pad = (pad == undefined) ? false : pad;
        this.init(margins, width, height, onSelectedCallBack, parentElement);

        this.line = d3.svg.line()
            .x(function (d) { return this.jMargin.left + this.dXScale(d["xValue"]) }.bind(this))
            .y(function (d) { return this.jMargin.top + this.dYScale(d["medianValue"]) }.bind(this));

        return this;
    }

    this.graphs.lineplot.prototype.display = function(id){
        if(this.toDisplay[id] != true) { 
            //console.log("Removing...", this.toDisplay[id]);
            this.svg.selectAll(".dataGroup_" + id).remove();
            this.svg.selectAll(".dataPath_" + id).remove();
            return;
        }

        //============================================
        groups = this.svg.selectAll(".dataGroup_" + id).data(this.data[id]);
        groups.enter().append("g").attr("class", "dataGroup_" + id);
        groups.exit().remove();

        //points = groups.selectAll(".subPoint").data(["minValue", "lowDevValue", "medianValue", "highDevValue", "maxValue"]);
        points = groups.selectAll(".subPoint").data(function(d, i, f){ 
            values =
                   [ {"cx": d["xValue"], "cy": d["lowDevValue"]},
                     {"cx": d["xValue"], "cy": d["medianValue"]},
                     {"cx": d["xValue"], "cy": d["highDevValue"]}] ;
            return values; });

        points.enter().append("circle").attr("class", "subPoint").on(this.mouseEvents[id]);
        points
            .attr("clip-path", "url(#" + this.id + "_clip)" )
            .attr("cx", function (d) { return this.jMargin.left + this.dXScale(d.cx) }.bind(this))
            .attr("cy", function (d) { return this.jMargin.top + this.dYScale(d.cy) }.bind(this))
            .attr(this.pointAttributes[id]);
        points.exit().remove();

        //============================================
     
        lines = groups.selectAll(".subline").data([""]);
        lines.enter().append("line").attr({"class":"subline", "stroke-width":"0.5", "stroke":this.pointAttributes[id].stroke});
        lines
            .attr("clip-path", "url(#" + this.id + "_clip)" )
            .attr("x1", function (d, e, f) { return this.jMargin.left + this.dXScale(this.data[id][f]["xValue"]); }.bind(this))
            .attr("x2", function (d, e, f) { return this.jMargin.left + this.dXScale(this.data[id][f]["xValue"]); }.bind(this))
            .attr("y1", function (d, e, f) { return this.jMargin.top + this.dYScale(this.data[id][f]["lowDevValue"]); }.bind(this))
            .attr("y2", function (d, e, f) { return this.jMargin.top + this.dYScale(this.data[id][f]["highDevValue"]); }.bind(this));
        lines.exit().remove();

        //=============================================

        paths = this.svg.selectAll(".dataPath_" + id).data([this.data[id]]);

        if (this.data[id].length > 1) {
            paths.enter()
                .append("path")
                .attr("class", "dataPath_" + id)
                .attr("stroke", this.pointAttributes[id].stroke)
                .attr("stroke-width", 1);

            paths
                .attr("clip-path", "url(#" + this.id + "_clip)" )
                .attr("d", function (d) { return this.line(d) }.bind(this));

            paths.exit().remove();
        }else{
            paths.remove();
        }
    }

    this.graphs.lineplot.prototype.updateDataFeatures = function(){
        this.xMin = this.xMax = this.yMin = this.yMax = undefined;
        Object.keys(this.data).forEach(function(id){
            if(this.toDisplay[id] != true){ return }
            xRange = d3.extent(this.data[id], function(d) { return d["xValue"] });
            this.xMin = (this.xMin == undefined) ? xRange[0] : d3.min([this.xMin, xRange[0]]);
            this.xMax = (this.xMax == undefined) ? xRange[1] : d3.max([this.xMax, xRange[1]]);

            yMin = d3.min(this.data[id], function(d) { return d["lowDevValue"] });
            yMax = d3.max(this.data[id], function(d) { return d["highDevValue"] });

            yRange = d3.extent(this.data[id], function(d) { return d.value});
            this.yMin = (this.yMin == undefined) ? yMin : d3.min([this.yMin, yMin]);
            this.yMax = (this.yMax == undefined) ? yMax : d3.max([this.yMax, yMax]);
        }.bind(this));

        this.updateDomains([this.xMin, this.xMax], [this.yMin, this.yMax]);
    }

    this.graphs.lineplot.prototype.updateDomains = function(xDomain, yDomain){
        var xValLeft, xValRight;
        range = xDomain[1] - xDomain[0];
        if(this.pad && range > 0){
            pad = 0.1 * range;
            xValLeft = xDomain[0] - pad;
            xValRight = xDomain[1] + pad;
        }
        else if (this.pad){
            pad = xDomain[1] > 10 ? 10 : this.xMax;
            xValLeft = xDomain[0] - pad;
            xValRight = xDomain[0] + pad;
        }else{
            xValLeft = xDomain[0];
            xValRight = xDomain[1];
        }
        

        //console.log("Range Used: ", xValLeft, xValRight);
        this.dXScale.range([0, this.width]).domain([xValLeft, xValRight]);
        this.dYScale.range([this.height, 0]).domain([0, this.yMax]);

        this.dYAxis.scale(this.dYScale);
        this.svg.select(".y.axis").call(this.dYAxis);

        this.dXAxis.scale(this.dXScale);
        this.svg.select(".x.axis").call(this.dXAxis);
    }

    console.log("Prototype:", this.graphs.base.prototype);
    this.graphs.lineplot.prototype = Object.create(this.graphs.base.prototype);
    this.graphs.lineplot.prototype.constructor = this.graphs.lineplot;

    //--------------------------------
    // Time Plot Magnitude
    // -------------------------------

    this.graphs.timeplot = function(margins, width, height, selectedCallBack, parentElement, displayPoints){
        safe.graphs.base.apply(this, [margins, width, height, selectedCallBack, parentElement]);
        console.log("Time Plot:", this, this.selectedCallBack);

        this.displayPoints = displayPoints == undefined ? true : displayPoints;

        this.line = d3.svg.line()
            .x(function(d) { return this.jMargin.left + this.dXScale(d.time) }.bind(this) )
            .y(function(d) { return this.jMargin.top + this.dYScale(d.value) }.bind(this) );

    }

    console.log("Prototype:", this.graphs.base.prototype);
    this.graphs.timeplot.prototype = Object.create(this.graphs.base.prototype);
    console.log("Created Prototype:", this.graphs.timeplot.prototype);
    this.graphs.timeplot.prototype.constructor = this.graphs.timeplot;

    this.graphs.timeplot.prototype.display = function(id){
        if(this.toDisplay[id] != true) { 
            this.svg.selectAll(".circle_" + id).remove();
            this.svg.selectAll(".path_" + id).remove();
            return;
        }

        var paths = this.svg.selectAll(".path_" + id).data([this.data[id]]);

        paths.enter().append("path")

        paths
            .attr("d", function(d){ return this.line(d) }.bind(this))
            .attr("clip-path", "url(#" + this.id + "_clip)" )
            .attr("class", "path_" + id)
            .attr({"stroke-width":"0.5", "stroke":this.pointAttributes[id].stroke });

        paths.exit().remove();

        if(this.displayPoints == true){
            var circles = this.svg.selectAll(".circle_" + id).data(this.data[id]);
            circles.enter().append("circle").attr("class", "circle_" + id).attr(this.pointAttributes[id])
                .attr("clip-path", "url(#" + this.id + "_clip)" );
            
            var counter = 0;
            circles.each(function(d) { if(d.time > this.dXScale.domain()[0] && d.time < this.dXScale.domain()[1]) counter++; }.bind(this));
            var scaleFactor = this.getScaleFactor(10, counter);
            var position = 0;

            circles 
                .attr("cx", function(d) { return this.jMargin.left + this.dXScale(d.time) }.bind(this))
                .attr("cy", function(d) { return this.jMargin.top + this.dYScale(d.value) }.bind(this))
                .attr("r", 1.5)
                .attr(this.pointAttributes[id])
                .attr("visibility", function(d, i){
                    var t;
                    if((i % scaleFactor == 0) && this.dXScale(d.time) >= position) {
                        t = "visible";
                        position = this.dXScale(d.time) + 5;
                    } else {
                        t = "hidden";
                    }
                    return t;
                    }.bind(this))
                .on(this.mouseEvents[id]);

           circles.exit().remove();
        }
    }

    this.graphs.timeplot.prototype.addDataSet = function(id, data, pointAttributes, mouseEvents){
        data.sort(function(a, b){ return a.time - b.time });
        this.data[id] = data;
        this.pointAttributes[id] = pointAttributes;
        this.mouseEvents[id] = mouseEvents;
        this.toDisplay[id] = true;
        this.updateDataFeatures();
    }

    this.graphs.timeplot.prototype.getScaleFactor = function(pointsPerPixel, size) {
        return Math.ceil((size / this.width) * pointsPerPixel);
    }



}).call(safe);
