// Layout components
var jMarginFocus, sFocusWidth, sContextWidth, sFocusHeight, sContextHeight;
// Scaling 
var dXScaleFocus, dYScaleFocus, dXScaleContext, dYScaleContext;
// Axes to be used for the graph
var xAxisFocus, yAxisFocus, xAxisContext, yAxisContext;
// Areas where things will be graphed
var svg, context, focus, brush;
// Our data, this will need to change
var designPointList, factorList,  runList, metricList, metricData, aDisplayList, levelList, parameterList;
// Inportant details about the data
var xValMin, xValMax, yValMin, yValMax;
// The name of the independent variable / dependent variables (Array)
var aDisplayList, aDisplayColor, aColors;
// Drawing details
var contextDataPointRadius, focusDataPointRadius;
// Used to display data information
var infoMessage, numberFormat;
var exp_id;

// Initializes graph componenets and variables
function initVisualComponents(){
	// Configure margins
	jMarginFocus = {top: 10, right: 40, bottom: 20, left: 40};
	marginContext = { top: 10, right: jMarginFocus.right, bottom: 20, left: 40 };
	sFocusWidth = 960 - jMarginFocus.left - jMarginFocus.right;
	sFocusHeight = 500 - jMarginFocus.top - jMarginFocus.bottom;
	sContextWidth = 500 - marginContext.left - marginContext.right;
	sContextHeight = sFocusHeight;

	// Configure time scales
	dXScaleFocus = d3.scale.linear().range([0, sFocusWidth]).domain([0,0]).clamp(true);
	dYScaleFocus = d3.scale.linear().range([sFocusHeight, 0]).domain([0,0]).clamp(true);
	dXScaleContext = d3.scale.linear().range([0, sContextWidth]).domain([0, 0]);
	dYScaleContext = d3.scale.linear().range([sContextHeight, 0]).domain([0, 0]);

	// Configure axis
	xAxisFocus = d3.svg.axis().scale(dXScaleFocus).orient("bottom");
	yAxisFocus = d3.svg.axis().scale(dYScaleFocus).orient("left");
	xAxisContext = d3.svg.axis().scale(dXScaleContext).orient("bottom");
	yAxisContext = d3.svg.axis().scale(dYScaleContext).orient("left");

    // Add focus plot
	focus = d3.select("#chart1")
        .append("svg")
        .attr("width", sFocusWidth + jMarginFocus.left + jMarginFocus.right)
        .attr("height", sFocusHeight + jMarginFocus.top + jMarginFocus.bottom + 30);

    // Add context plot
    context = d3.select("#chart2")
        .append("svg")
        .attr("width", sContextWidth + marginContext.left + marginContext.right)
        .attr("height", sContextHeight + marginContext.top + marginContext.bottom + 30)

	// Text label for x axis
	focus.append("text")
        .attr("class", "x label")
		.attr("x", sFocusWidth / 2)
		.attr("y", sFocusHeight + jMarginFocus.top + jMarginFocus.bottom)
		.attr("dy", "3em")
		.style("text-anchor", "middle")
		.text("Original x-axis");

	// Add text label for y axis
	focus.append("text")
		.attr("class", "y label")
		.attr("transform", "rotate(-90)")
		.attr("y", 0)
		.attr("x", 0 - ((sFocusHeight) / 2))
		.attr("dy", "1em")
		.style("text-anchor", "middle")
		.text("Original y-axis");

	// Add x axis
	focus.append("g")
		.attr("class", "x axis focus")
		.attr("transform", "translate(" + jMarginFocus.left + "," + (sFocusHeight + jMarginFocus.top) + ")")
		.call(xAxisFocus);

	context.append("g")
        .attr("class", "x axis context")
        .attr("transform", "translate(" + marginContext.left + "," + (sContextHeight + marginContext.top) + ")")
        .call(xAxisContext);

	// Add y axis
    focus.append("g")
		.attr("class", "y axis focus")
		.attr("transform", "translate(" + jMarginFocus.left + "," + jMarginFocus.top + ")")
		.call(yAxisFocus);

    context.append("g")
		.attr("class", "y axis context")
		.attr("transform", "translate(" + marginContext.left + "," + marginContext.top + ")")
		.call(yAxisContext);

    brush = d3.svg.brush() 
        .x(dXScaleContext)
        .y(dYScaleContext)
        .on("brush", brushed);

    context.append("g")
        .attr("class", "brush")
        .attr("transform", "translate(" + marginContext.left + "," + marginContext.top + ")")
        .call(brush);

    // Add the information bar
	infoMessage = d3.select("body")
        .append("div")
		.attr("class", "tooltip")
        .style("opacity", 0);

	// Miscilanious variables
	aDisplayColor = {};
    aDisplayList = {};
	metricData = {};
    aColors = ["blue", "green", "orange", "purple", "black", "red"];
	contextDataPointRadius = 1.5;
	focusDataPointRadius = 3;
	parameterList = {};
    numberFormat = d3.format(".2f");
}

// Initializes the selection for the graph
function initGraphSelect(){
    // Initialize the factor list
    d3.selectAll("#select")
        .append("div")
        .append("select")
        .attr("class", "key_line")
        .attr("id", "selectFactor")
        .on("change", factorSelectChange);

    d3.select("#selectFactor")
        .selectAll("option")
        .data(Object.keys(factorList))
        .enter()
        .append("option")
        .attr("factor", function(d) { return d } )
        .text(function(d) { return factorList[d].name });
    
    // Initialize the level list

	var rangeDiv = d3.selectAll("#select")
		.append("div")

    rangeDiv
        .append("select")
		.style({"display": "inline-block", "width":"50%"})
        .attr("class", "key_line")
        .attr("id", "selectLevelMin")

    rangeDiv
        .append("select")
		.style({"display": "inline-block", "width":"50%"})
        .attr("id", "selectLevelMax")
    
    // Initialize the apply selection button
    d3.selectAll("#select")
        .append("button")
        .text("Apply Selection Filter")
        .on("click", applySelectionFilter);
    
    
    // Initialize the design point list
    d3.select("#select")
        .append("div")
        .append("select")
        .attr("class", "key_line")
        .attr("id", "selectDesignPoint")
        .on("change", designPointSelectChange)

    
    // Initialize the run list
    d3.selectAll("#select")
        .append("div")
        .append("select")
        .attr("class", "key_line")
        .attr("id", "selectRun")
        .on("change", runSelectChange)
    
    // Initialize metric selects
    var select_items = d3.select("#select").selectAll(".select_line").data(Object.keys(metricList)).enter()
        .append("div")
        .attr("class", "select_line")
        .attr("metric_id", function(d) { return metricList[d].id })
        .attr("id", function(d) { return "select_" + metricList[d].id });

    select_items.append("div")
        .attr("class", "select_square")
        .attr("id", function(d) { return "select_square_" + metricList[d].id });

    select_items.append("div")
        .attr("class", "select_label")
        .attr("id", function(d) { return "select_label_" + metricList[d].id })
        .text(function(d) { return metricList[d].name});

    d3.selectAll(".select_line")
        .on("click", selectElementChange);

    factorSelectChange.call(document.getElementById("selectFactor"));
    updateDesignPoints();

    // Initialize the cross hairs option
    var crossHairEnable = d3.selectAll("#options")
        .append("div")
        .attr("id", "crossHairEnable");

	crossHairEnable.append("input")
        .attr("type", "checkbox")
        .style("float", "left")
        .on("change", enableCrossHairs);

	crossHairEnable.append("p")
        .text("Cross Hairs");
}

// Reads json data from the given url
// When it's loaded the given callBack function
// is called with the resulting data
function readData(url, callBack){
    d3.json(url, function(error, json){
        if (error) return console.warn(error);

        callBack(json);
    });
}

// Reads metric information
function readInitialData(){

    readData("/safe/exp/" + exp_id + "/designpoints/", function(data){
        designPointList = [];
        runList = [];
        data.forEach(function(designPoint){ designPointList[designPoint.id] = designPoint});
        data.forEach(function(designPoint){ readData("/safe/designpoint/" + designPoint.id + "/simruns/", function(data){
                runList.concat(data);
                designPoint["runs"] = data;
            });
            });

        readData("/safe/exp/" + exp_id + "/factors/", function(data){
            factorList = {};
            levelList = {};
            data.forEach(function(factor){ factorList[factor.id] = factor });
            readData("/safe/exp/" + exp_id + "/levels/", function(data){
                Object.keys(factorList).forEach(function(factor_id){
                    levelList[factor_id] = data[factor_id].levels.sort();
                  });
                metricList = {};
                readData("/safe/exp/" + exp_id + "/metrics/", function(data){
                    data.forEach(function(element){
                        metricList[element.id] = element;
                      });
                    initGraphSelect();
                    updateDataFeatures();
                    displayData();
                  });
              });
            
        });
    });
}


// displays the currently held data to the graph
function displayData(){
	Object.keys(aDisplayList).forEach(function (run_id) {
            Object.keys(aDisplayList[run_id]).forEach(function(metric_id){
                updateContextDataPoints(run_id, metric_id);
            });
	});
}
// updates the positions of the center points described by the data groupings
function updateContextDataPoints(run_id, metric_id){
    contextpoints = context.selectAll(".contextDataPoint_" + run_id + "_" + metric_id).data(metricData[run_id][metric_id]);

    contextpoints.enter()
        .append("circle")
        .attr("class", "contextDataPoint_" + run_id + "_" + metric_id)
        .attr("stroke", aDisplayColor[run_id][metric_id])
        .attr("fill", "none")
        .attr("r", contextDataPointRadius);

    contextpoints.transition()
        .attr("cx", function (d) { return marginContext.left + dXScaleContext(d.time) })
        .attr("cy", function (d) { return marginContext.top + dYScaleContext(d.value) });

	contextpoints.exit()
        .remove();	
}

function updateFocusDataPoints(run_id, metric_id){
    currentData = [];

    metricData[run_id][metric_id].forEach(function (d) {
        if (dXScaleFocus.domain()[0] <= d.time && d.time <= dXScaleFocus.domain()[1])
            if (dYScaleFocus.domain()[0] <= d.value && d.value <= dYScaleFocus.domain()[1])
                currentData.push(d);
    });

    focuspoints = focus.selectAll(".focusDataPoint_" + run_id + "_" + metric_id).data(currentData);
    focuspoints.enter()
        .append("circle")
        .attr("class", "focusDataPoint_" + run_id + "_" + metric_id)
        .attr("stroke", aDisplayColor[run_id][metric_id] )
        .attr("stroke-width", 1.5)
        .attr("fill", "white")
        .attr("fill-opacity", 0)
        .attr("r", focusDataPointRadius)
        .attr("info", function (d) { return numberFormat(d.time) + "<br/>" + numberFormat(d.value) } )
        .on("mouseover", mouseOverDataPoint)
        .on("mouseout", mouseOutDataPoint);

    focuspoints
        .attr("cx", function (d) { return jMarginFocus.left + dXScaleFocus(d.time)})
        .attr("cy", function (d) { return jMarginFocus.top + dYScaleFocus(d.value) })
        .attr("info", function (d) { return numberFormat(d.time) + "<br/>" + numberFormat(d.value)} );

    focuspoints.exit()
        .remove();
}

function addDisplayMetric(run_id, metric_id){
    var eRun = document.getElementById("selectRun")
    var sRun = eRun.options[eRun.selectedIndex].getAttribute("run");
    if(run_id == sRun){ d3.selectAll("#select_square_" + metric_id).style("background", aDisplayColor[run_id][metric_id]); }

    aDisplayList[run_id][metric_id] = true;
        
    if(metricData[run_id] == undefined){ metricData[run_id] = {} }
    if(metricData[run_id][metric_id] == undefined){
        readData("/safe/simrun/" + run_id + "/metric/" + metric_id + "/results/", function(data){
            metricData[run_id][metric_id] = data;
            updateDataFeatures();
            updateScaling();
            displayData();
			updateKeyItems();
        });
    }else{
        updateDataFeatures();
        updateScaling();
        displayData();
		updateKeyItems();
    }
}

// removes the specified key fro the list of dependent variables (no longer displayed)
function removeDisplayMetric(run_id, metric_id){
    var eRun = document.getElementById("selectRun")
    var sRun = eRun.options[eRun.selectedIndex].getAttribute("run");
    if(run_id == sRun){ d3.selectAll("#select_square_" + metric_id).style("background", "white"); }
	
    delete aDisplayList[run_id][metric_id];
    context.selectAll(".contextDataPoint_" + run_id + "_" + metric_id).remove();
	focus.selectAll(".focusDataPoint_" + run_id + "_" + metric_id).remove();
    updateDataFeatures();
    updateScaling();
    displayData();
	updateKeyItems();
}

// called when the mouse is positioned over a data element
function mouseOverDataPoint(){
	infoMessage.transition()        
                .duration(200)      
                .style("opacity", .9);
    infoMessage.html(this.getAttribute("info"))  
        .style("left", (d3.event.pageX) + "px")     
        .style("top", (d3.event.pageY - 28) + "px");
}

// called when the mouse is no longer over a data element
function mouseOutDataPoint(){
	infoMessage.transition()
        .duration(500)
        .style("opacity", 0);
}



// adjusts the scales base on current data values, dependent, and independent variables
function updateScaling(){
	dXScaleContext.range([0, sContextWidth]).domain([0,xValMax]);
	dYScaleContext.range([sContextHeight, 0]).domain([0,yValMax]);

	yAxisContext.scale(dYScaleContext);
	d3.select(".y.axis.context").transition().call(yAxisContext);

	xAxisContext.scale(dXScaleContext);
    d3.select(".x.axis.context").transition().call(xAxisContext);

    brushed();
}

// updates important information (such as min/max values)
function updateDataFeatures(){
    xValMin = -1;
    xValMax = -1;
    yValMin = -1;
    yValMax = -1;
    Object.keys(aDisplayList).forEach(function (run_id) {
        Object.keys(aDisplayList[run_id]).forEach(function(metric_id){
            var currentDisplayList = metricData[run_id][metric_id];
            if (currentDisplayList != undefined) {
                xValRange = d3.extent(metricData[run_id][metric_id], function (d) { return d.time; });
                xValMin = (xValMin < 0) ? xValRange[0] : d3.min([xValMin, xValRange[0]]);
                xValMax = (xValMax < 0) ? xValRange[1] : d3.max([xValMax, xValRange[1]]);

                yValRange = d3.extent(metricData[run_id][metric_id], function (d) { return d.value; });
                yValMin = (yValMin) < 0 ? yValRange[0] : d3.min([yValMin, yValRange[0]]);
                yValMax = (yValMax) < 0 ? yValRange[1] : d3.max([yValMax, yValRange[1]]);
            }
        });
    });
    if (d3.sum(aDisplayList, function(d){ return d.length } == 0))
        xValMin = xValMax = yValMin = yValMax = 0;
}

function brushed() {
    var extent = brush.extent();
    dXScaleFocus.domain(brush.empty() ? dXScaleContext.domain() : [ extent[0][0], extent [1][0] ]);
    dYScaleFocus.domain(brush.empty() ? dYScaleContext.domain() : [ extent[0][1], extent [1][1] ]);

    focus.select(".x.axis.focus").call(xAxisFocus);
    focus.select(".y.axis.focus").call(yAxisFocus);

    Object.keys(aDisplayList).forEach(function(run_id){
          Object.keys(aDisplayList[run_id]).forEach(function(metric_id){
                updateFocusDataPoints(run_id, metric_id);
        });
    });
}

function updateCrossHairs(){
    var x = d3.event.offsetX;
    var y = d3.event.offsetY;

    var xleft = dXScaleFocus.range()[0] + jMarginFocus.left; 
    var xright = dXScaleFocus.range()[1] + jMarginFocus.left;
    var ytop = dYScaleFocus.range()[0] + jMarginFocus.top;
    var ybottom = dYScaleFocus.range()[1] + jMarginFocus.top;

    if (x < xleft) { x = xleft }
    if (x > xright) { x = xright }
    if (y < ybottom) { y = ybottom }
    if (y > ytop) { y = ytop }

    horizontal = focus.selectAll(".crosshairhorizontal").data([""]);
    vertical = focus.selectAll(".crosshairvertical").data([""]);

    horizontal.attr("x1", xleft).attr("x2", xright).attr("y1", y).attr("y2", y);
    vertical.attr("y1", ybottom).attr("y2", ytop).attr("x1", x).attr("x2", x);

    horizontal.enter().append("line").attr("class", "crosshairhorizontal").style("stroke", "black").style("pointer-events", "none")
        .attr("x1", xleft).attr("x2", xright).attr("y1", y).attr("y2", y);
    vertical.enter().append("line").attr("class", "crosshairvertical").style("stroke", "black").style("pointer-events", "none")
        .attr("y1", ybottom).attr("y2", ytop).attr("x1", x).attr("x2", x);

    horizontal.exit().remove();
    vertical.exit().remove();
}

function enableCrossHairs(){
    if(this.checked){
        focus.on("mousemove", updateCrossHairs);
    }else{
        focus.selectAll(".crosshairhorizontal").remove();
        focus.selectAll(".crosshairvertical").remove();
        focus.on("mousemove", undefined);
    }
}

function factorSelectChange(){
    console.log("Factor changed...");
    var option = this.options[this.selectedIndex];
    var levelSelectMin = document.getElementById("selectLevelMin");
    var levelSelectMax = document.getElementById("selectLevelMax");

    factor = option.getAttribute("factor");

	levels = d3.select(levelSelectMin).selectAll("option").data(levelList[factor]);
	levels.enter().append("option");
	levels.attr("level", function(d) { return d }).text(function(d) { return d });
	levels.exit().remove();

    levels = d3.select(levelSelectMax).selectAll("option").data(levelList[factor]);
	levels.enter().append("option");
	levels.attr("level", function(d) { return d }).text(function(d) { return d });
	levels.exit().remove();
	levelSelectMax.selectedIndex = levelList[factor].length - 1;

}

function applySelectionFilter(){
    console.log("Level changed...");
    var designPointSelect = document.getElementById("selectDesignPoint");
    var factorSelect = document.getElementById("selectFactor");
    var levelMinSelect = document.getElementById("selectLevelMin");
    var levelMaxSelect = document.getElementById("selectLevelMax");

    var factor = factorList[factorSelect.options[factorSelect.selectedIndex].getAttribute("factor")];
    var levelMin = levelMinSelect.options[levelMinSelect.selectedIndex].getAttribute("level");
    var levelMax = levelMaxSelect.options[levelMaxSelect.selectedIndex].getAttribute("level");
    
    if(levelMin > levelMax){
        alert("The factor's beginning value cannot be greater than the factor's ending value");
		return;
    }

	console.log(levelMinSelect.selectedIndex, levelMaxSelect.selectedIndex, levelList);
	if(levelMinSelect.selectedIndex > 0 || levelMaxSelect.selectedIndex < levelList[factor.id].length - 1){
		parameterList[factor.id] = {"name": factor.name, "min_value":levelMin, "max_value":levelMax};
	}else{
		delete parameterList[factor.id];
	}
	console.log(parameterList, factor.id, levelMin, levelMax);

    updateDesignPoints();
}

function updateDesignPoints(){
	parameters = [];
	Object.keys(parameterList).forEach(function(parameter){ parameters.push(parameterList[parameter]) });
	console.log(parameters);

    var designPointSelect = document.getElementById("selectDesignPoint");
    d3.json("/safe/exp/" + exp_id + "/dp_range_query/")
			.post(JSON.stringify(parameters), function(error, data){
				if(error){ console.ward(error); }
				
				console.log(data);
	            d3.select(designPointSelect).selectAll("option").remove();
	            designPoints = d3.select(designPointSelect).selectAll("option").data(data);
	            designPoints.enter().append("option");
	            designPoints.attr("designPoint", function(d) { return d }).text(function(d) { return "Design Point " + d});
	            designPointSelectChange.call(designPointSelect);
                updateSelectionItems();
        });
}


function designPointSelectChange(){
    console.log("Design Point changed...");
    var designPoint = designPointList[this.options[this.selectedIndex].getAttribute("designPoint")];
    readData("/safe/designpoint/" + designPoint.id + "/simruns/", function(data){
        runs = d3.select("#selectRun").selectAll("option").data(data);
        runs.enter().append("option")
        runs.attr("run", function(d) { return d.simrun_id }).text(function(d) { return "Run " + d.simrun_id });
        runs.exit().remove();
        // Prepend aggregate run here
        });
}

function runSelectChange(){
  console.log("Run changed...");
    var sSelectedRun = this.selectedOptions[0].getAttribute("run");
    Object.keys(metricList).forEach(function(sSelectedMetric){
	if(aDisplayList[sSelectedRun] != undefined && aDisplayList[sSelectedRun][sSelectedMetric] != undefined){
            d3.select("#select_square_" + sSelectedMetric).style("background", aDisplayColor[sSelectedRun][sSelectedMetric]);
    	}else{
            d3.select("#select_square_" + sSelectedMetric).style("background", "white");
	}
    });
}

function selectElementChange(){
     console.log("Selected Element: ", this.getAttribute("metric_id"));
    
     var dSelectedRun = d3.select("#selectRun")[0][0];
     var sSelectedRun = dSelectedRun.selectedOptions[0].getAttribute("run");
     var sSelectedMetric = this.getAttribute("metric_id");

     if(aDisplayColor[sSelectedRun] == undefined){ aDisplayColor[sSelectedRun] = {} }
     if(aDisplayList[sSelectedRun] == undefined) { aDisplayList[sSelectedRun] = {} }

     if(aDisplayColor[sSelectedRun][sSelectedMetric] == undefined) { aDisplayColor[sSelectedRun][sSelectedMetric] = aColors.shift() }

     if(aDisplayList[sSelectedRun][sSelectedMetric] == undefined){
         addDisplayMetric(sSelectedRun, sSelectedMetric);
     }else{
         removeDisplayMetric(sSelectedRun, sSelectedMetric);
     }
}

function updateKeyItems(){
	var values = [];
	Object.keys(aDisplayList).forEach(function(run_id){
		Object.keys(aDisplayList[run_id]).forEach(function(metric_id){
			values.push({"run_id":run_id, "metric_id": metric_id});
		});		
	});

	values.sort(function(a, b){ return a.run_id - b.run_id });
	d3.select("#key").selectAll(".key_line").remove();

	var displayed = d3.select("#key").selectAll(".key_line").data(values)
			.enter()
			.append("div")
			.attr("class", "key_line")

    displayed.append("div")
        .attr("run_id", function(d) { console.log(d); return d.run_id })
		.attr("metric_id", function(d) { return d.metric_id })
        .style("float", "left")
        .text("X")
        .on("click", displayRemove);

	displayed.append("div")
		.attr("class", "key_square")
		.style("background", function(d){ return aDisplayColor[d.run_id][d.metric_id] });

	displayed.append("div")
		.attr("class", "key_label")
		.text(function(d) { return "Run: " + d.run_id + ", Metric: " + d.metric_id });
}

function displayRemove(){
    console.log("Removing ", this, "...");
    removeDisplayMetric(this.getAttribute("run_id"), this.getAttribute("metric_id"));
}

function updateSelectionItems(){
	console.log("Updating Selection...");
    d3.select("#parameters").selectAll(".select_line").remove();

    var keys = Object.keys(parameterList).sort();
    var parameters = d3.select("#parameters").selectAll(".select_line").data(keys)
        .enter()
        .append("div")
        .attr("class", "select_line")

     parameters
        .append("div")
        .attr("factor_id", function(d) { return d })
        .style("float", "left")
        .text("X : ")
        .on("click", function(){
                delete parameterList[this.getAttribute("factor_id")];
                updateDesignPoints();
                updateSelectionItems();
                });

    parameters
        .append("div")
        .attr("class", "select_label")
        .text(function(d) { return "Factor: " + parameterList[d].name + ",   Range: " + parameterList[d].min_value + " - " + parameterList[d].max_value });
    console.log(parameterList);
}

function readParameters(){
    var parameterList = location.search.substring(1).split("&");
    var parameters = {};
    parameterList.forEach(function(parameter){
        var temp = parameter.split("=");
        parameters[temp[0]] = temp[1];
        });

    exp_id = parameters["exp_id"];
}

readParameters();
initVisualComponents();
readInitialData();
